@@ -2,7 +2,7 @@
 /*!
   @file    exv_msvc.h
   @brief   Configuration settings for MSVC
-  @version $Rev: 1834 $
+  @version $Rev: 2451 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    07-Feb-04, ahu: created
@@ -23,34 +23,31 @@
 #define EXV_PACKAGE_NAME "exiv2"
 
 /* Define to the full name and version of this package. */
-#define EXV_PACKAGE_STRING "exiv2 0.18.2"
+#define EXV_PACKAGE_STRING "exiv2 0.21.1"
 
 /* Define to the one symbol short name of this package. */
 #define EXV_PACKAGE_TARNAME "exiv2"
 
 /* Define to the version of this package. */
-#define EXV_PACKAGE_VERSION "0.18.2"
+#define EXV_PACKAGE_VERSION "0.21.1"
 
 /* Define to `int' if <sys/types.h> does not define pid_t. */
 typedef int pid_t;
 
 #ifndef EXV_COMMERCIAL_VERSION
 
-/* Define to 1 to enable translation of Exif.Nikon3.LensData. */
+/* Define to 1 to enable translation of Nikon lens names. */
 # define EXV_HAVE_LENSDATA 1
 
 /* Define to 1 if translation of program messages to the user's
    native language is requested. */
 # undef EXV_ENABLE_NLS
 
+#endif /* !EXV_COMMERCIAL_VERSION */
+
 /* Define to 1 if you have the `iconv' function. */
 # undef EXV_HAVE_ICONV
 
-/* Define to 1 to enable conversion of UCS2 encoded Windows tags to UTF-8. */
-# undef EXV_HAVE_PRINTUCS2
-
-#endif /* !EXV_COMMERCIAL_VERSION */
-
 /* Define as 1 if you have the `zlib' library. (0 to omit zlib) [png support] */
 #define HAVE_LIBZ 1
 
@@ -69,6 +66,14 @@ typedef int pid_t;
 #define EXV_SEPERATOR_STR "\\"
 #define EXV_SEPERATOR_CHR '\\'
 
+/* Windows unicode path support */
+#define EXV_UNICODE_PATH
+
+/* Define to 1 if you have the `mmap' function. */
+/* #undef EXV_HAVE_MMAP */
+
+/* Define to 1 if you have the `munmap' function. */
+/* #undef EXV_HAVE_MUNMAP */
 
 /* Shared library support */
 #ifdef  EXV_HAVE_DLL
