@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      actions.cpp
-  Version:   $Rev: 1760 $
+  Version:   $Rev: 2455 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
              30-Apr-06, Roger Larsson: Print filename if processing multiple files
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: actions.cpp 1760 2009-03-07 15:26:00Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: actions.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -41,7 +41,7 @@ EXIV2_RCSID("@(#) $Id: actions.cpp 1760 2009-03-07 15:26:00Z ahuggel $")
 # include "timegm.h"
 #endif
 #include "actions.hpp"
-#include "exiv2.hpp"
+#include "exiv2app.hpp"
 #include "image.hpp"
 #include "jpgimage.hpp"
 #include "xmpsidecar.hpp"
@@ -49,7 +49,6 @@ EXIV2_RCSID("@(#) $Id: actions.cpp 1760 2009-03-07 15:26:00Z ahuggel $")
 #include "types.hpp"
 #include "exif.hpp"
 #include "easyaccess.hpp"
-#include "canonmn.hpp"
 #include "iptc.hpp"
 #include "xmp.hpp"
 #include "preview.hpp"
@@ -128,12 +127,6 @@ namespace {
                  int targetType,
                  bool preserve);
 
-    /*!
-      @brief Test to distinguish between Exif and other tags in TIFF-like files.
-             Only Exif tags are copyed, deleted, added.
-     */
-    bool isExifTag(const Exiv2::Exifdatum& ed);
-
     /*!
       @brief Rename a file according to a timestamp value.
 
@@ -167,6 +160,10 @@ namespace {
 // class member definitions
 namespace Action {
 
+    Task::~Task()
+    {
+    }
+
     Task::AutoPtr Task::clone() const
     {
         return AutoPtr(clone_());
@@ -214,6 +211,7 @@ namespace Action {
         registerTask(insert,  Task::AutoPtr(new Insert));
         registerTask(modify,  Task::AutoPtr(new Modify));
         registerTask(fixiso,  Task::AutoPtr(new FixIso));
+        registerTask(fixcom,  Task::AutoPtr(new FixCom));
     } // TaskFactory c'tor
 
     Task::AutoPtr TaskFactory::create(TaskType type)
@@ -226,6 +224,10 @@ namespace Action {
         return Task::AutoPtr(0);
     } // TaskFactory::create
 
+    Print::~Print()
+    {
+    }
+
     int Print::run(const std::string& path)
     try {
         path_ = path;
@@ -293,7 +295,7 @@ namespace Action {
 
         // Image number
         // Todo: Image number for cameras other than Canon
-        printTag(exifData, "Exif.Canon.ImageNumber", _("Image number"));
+        printTag(exifData, "Exif.Canon.FileNumber", _("Image number"));
 
         // Exposure time
         // From ExposureTime, failing that, try ShutterSpeedValue
@@ -375,19 +377,25 @@ namespace Action {
         printLabel(_("Exif Resolution"));
         long xdim = 0;
         long ydim = 0;
-        md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageWidth"));
-        if (md == exifData.end()) {
-            md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
+        if (image->mimeType() == "image/tiff") {
+            xdim = image->pixelWidth();
+            ydim = image->pixelHeight();
         }
-        if (md != exifData.end() && md->count() > 0) {
-            xdim = md->toLong();
-        }
-        md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"));
-        if (md == exifData.end()) {
-            md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
-        }
-        if (md != exifData.end() && md->count() > 0) {
-            ydim = md->toLong();
+        else {
+            md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageWidth"));
+            if (md == exifData.end()) {
+                md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
+            }
+            if (md != exifData.end() && md->count() > 0) {
+                xdim = md->toLong();
+            }
+            md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"));
+            if (md == exifData.end()) {
+                md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
+            }
+            if (md != exifData.end() && md->count() > 0) {
+                ydim = md->toLong();
+            }
         }
         if (xdim != 0 && ydim != 0) {
             std::cout << xdim << " x " << ydim;
@@ -473,16 +481,14 @@ namespace Action {
 
     int Print::printList()
     {
-        int rc = 0;
         if (!Exiv2::fileExists(path_, true)) {
             std::cerr << path_
                       << ": " << _("Failed to open the file\n");
-            rc = -1;
+            return -1;
         }
         Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
         assert(image.get() != 0);
         image->readMetadata();
-        bool const manyFiles = Params::instance().files_.size() > 1;
         // Set defaults for metadata types and data columns
         if (Params::instance().printTags_ == Exiv2::mdNone) {
             Params::instance().printTags_ = Exiv2::mdExif | Exiv2::mdIptc | Exiv2::mdXmp;
@@ -490,11 +496,17 @@ namespace Action {
         if (Params::instance().printItems_ == 0) {
             Params::instance().printItems_ = Params::prKey | Params::prType | Params::prCount | Params::prTrans;
         }
+        return printMetadata(image.get());
+    } // Print::printList
+
+    int Print::printMetadata(const Exiv2::Image* image)
+    {
+        int rc = 0;
         if (Params::instance().printTags_ & Exiv2::mdExif) {
-            Exiv2::ExifData& exifData = image->exifData();
+            const Exiv2::ExifData& exifData = image->exifData();
             for (Exiv2::ExifData::const_iterator md = exifData.begin();
                  md != exifData.end(); ++md) {
-                printMetadatum(*md, image.get(), manyFiles);
+                printMetadatum(*md, image);
             }
             if (exifData.empty()) {
                 if (Params::instance().verbose_) {
@@ -504,10 +516,10 @@ namespace Action {
             }
         }
         if (Params::instance().printTags_ & Exiv2::mdIptc) {
-            Exiv2::IptcData& iptcData = image->iptcData();
+            const Exiv2::IptcData& iptcData = image->iptcData();
             for (Exiv2::IptcData::const_iterator md = iptcData.begin();
                  md != iptcData.end(); ++md) {
-                printMetadatum(*md, image.get(), manyFiles);
+                printMetadatum(*md, image);
             }
             if (iptcData.empty()) {
                 if (Params::instance().verbose_) {
@@ -517,10 +529,10 @@ namespace Action {
             }
         }
         if (Params::instance().printTags_ & Exiv2::mdXmp) {
-            Exiv2::XmpData& xmpData = image->xmpData();
+            const Exiv2::XmpData& xmpData = image->xmpData();
             for (Exiv2::XmpData::const_iterator md = xmpData.begin();
                  md != xmpData.end(); ++md) {
-                printMetadatum(*md, image.get(), manyFiles);
+                printMetadatum(*md, image);
             }
             if (xmpData.empty()) {
                 if (Params::instance().verbose_) {
@@ -530,16 +542,27 @@ namespace Action {
             }
         }
         return rc;
-    } // Print::printList
+    } // Print::printMetadata
 
-    void Print::printMetadatum(const Exiv2::Metadatum& md,
-                               const Exiv2::Image* pImage,
-                               bool const manyFiles)
+    bool Print::grepTag(const std::string& key)
     {
+        if (Params::instance().keys_.empty()) return true;
+        for (Params::Keys::const_iterator k = Params::instance().keys_.begin();
+             k != Params::instance().keys_.end(); ++k) {
+            if (*k == key) return true;
+        }
+        return false;
+    }
+
+    void Print::printMetadatum(const Exiv2::Metadatum& md, const Exiv2::Image* pImage)
+    {
+        if (!grepTag(md.key())) return;
+
         if (   Params::instance().unknown_
             && md.tagName().substr(0, 2) == "0x") {
             return;
         }
+        bool const manyFiles = Params::instance().files_.size() > 1;
         if (manyFiles) {
             std::cout << std::setfill(' ') << std::left << std::setw(20)
                       << path_ << "  ";
@@ -608,30 +631,56 @@ namespace Action {
             if (!first) std::cout << "  ";
             first = false;
             if (   Params::instance().binary_
-                && md.typeId() == Exiv2::undefined
-                && md.size() > 100) {
+                && (   md.typeId() == Exiv2::undefined
+                    || md.typeId() == Exiv2::unsignedByte
+                    || md.typeId() == Exiv2::signedByte)
+                && md.size() > 128) {
                 std::cout << _("(Binary value suppressed)") << std::endl;
                 return;
             }
-            std::cout << std::dec << md.value();
+            bool done = false;
+            if (0 == strcmp(md.key().c_str(), "Exif.Photo.UserComment")) {
+                const Exiv2::CommentValue* pcv = dynamic_cast<const Exiv2::CommentValue*>(&md.value());
+                if (pcv) {
+                    Exiv2::CommentValue::CharsetId csId = pcv->charsetId();
+                    if (csId != Exiv2::CommentValue::undefined) {
+                        std::cout << "charset=\"" << Exiv2::CommentValue::CharsetInfo::name(csId) << "\" ";
+                    }
+                    std::cout << pcv->comment(Params::instance().charset_.c_str());
+                    done = true;
+                }
+            }
+            if (!done) std::cout << std::dec << md.value();
         }
         if (Params::instance().printItems_ & Params::prTrans) {
             if (!first) std::cout << "  ";
             first = false;
             if (   Params::instance().binary_
-                && md.typeId() == Exiv2::undefined
-                && md.size() > 100) {
+                && (   md.typeId() == Exiv2::undefined
+                    || md.typeId() == Exiv2::unsignedByte
+                    || md.typeId() == Exiv2::signedByte)
+                && md.size() > 128) {
                 std::cout << _("(Binary value suppressed)") << std::endl;
                 return;
             }
-            std::cout << std::dec << md.print(&pImage->exifData());
+            bool done = false;
+            if (0 == strcmp(md.key().c_str(), "Exif.Photo.UserComment")) {
+                const Exiv2::CommentValue* pcv = dynamic_cast<const Exiv2::CommentValue*>(&md.value());
+                if (pcv) {
+                    std::cout << pcv->comment(Params::instance().charset_.c_str());
+                    done = true;
+                }
+            }
+            if (!done) std::cout << std::dec << md.print(&pImage->exifData());
         }
         if (Params::instance().printItems_ & Params::prHex) {
             if (!first) std::cout << std::endl;
             first = false;
             if (   Params::instance().binary_
-                && md.typeId() == Exiv2::undefined
-                && md.size() > 100) {
+                && (   md.typeId() == Exiv2::undefined
+                    || md.typeId() == Exiv2::unsignedByte
+                    || md.typeId() == Exiv2::signedByte)
+                && md.size() > 128) {
                 std::cout << _("(Binary value suppressed)") << std::endl;
                 return;
             }
@@ -699,6 +748,10 @@ namespace Action {
         return new Print(*this);
     }
 
+    Rename::~Rename()
+    {
+    }
+
     int Rename::run(const std::string& path)
     {
     try {
@@ -784,6 +837,10 @@ namespace Action {
         return new Rename(*this);
     }
 
+    Erase::~Erase()
+    {
+    }
+
     int Erase::run(const std::string& path)
     try {
         path_ = path;
@@ -852,13 +909,7 @@ namespace Action {
         if (Params::instance().verbose_ && image->exifData().count() > 0) {
             std::cout << _("Erasing Exif data from the file") << std::endl;
         }
-        if (0 == strcmp(image->mimeType().c_str(), "image/tiff")) {
-            Exiv2::ExifData& ed = image->exifData();
-            ed.erase(std::remove_if(ed.begin(), ed.end(), isExifTag), ed.end());
-        }
-        else {
-            image->clearExifData();
-        }
+        image->clearExifData();
         return 0;
     }
 
@@ -900,6 +951,10 @@ namespace Action {
         return new Erase(*this);
     }
 
+    Extract::~Extract()
+    {
+    }
+
     int Extract::run(const std::string& path)
     try {
         path_ = path;
@@ -1038,6 +1093,10 @@ namespace Action {
         return new Extract(*this);
     }
 
+    Insert::~Insert()
+    {
+    }
+
     int Insert::run(const std::string& path)
     try {
         if (!Exiv2::fileExists(path, true)) {
@@ -1137,6 +1196,10 @@ namespace Action {
         return new Insert(*this);
     }
 
+    Modify::~Modify()
+    {
+    }
+
     int Modify::run(const std::string& path)
     {
     try {
@@ -1359,7 +1422,7 @@ namespace Action {
             std::cout << _("Reg ") << modifyCmd.key_ << "=\""
                       << modifyCmd.value_ << "\"" << std::endl;
         }
-        // Registration has been done immediately after parsing the command.
+        Exiv2::XmpProperties::registerNs(modifyCmd.value_, modifyCmd.key_);
     }
 
     Modify::AutoPtr Modify::clone() const
@@ -1372,6 +1435,10 @@ namespace Action {
         return new Modify(*this);
     }
 
+    Adjust::~Adjust()
+    {
+    }
+
     int Adjust::run(const std::string& path)
     try {
         adjustment_      = Params::instance().adjustment_;
@@ -1508,6 +1575,10 @@ namespace Action {
         return 0;
     } // Adjust::adjustDateTime
 
+    FixIso::~FixIso()
+    {
+    }
+
     int FixIso::run(const std::string& path)
     {
     try {
@@ -1529,37 +1600,15 @@ namespace Action {
                       << ": " << _("No Exif data found in the file\n");
             return -3;
         }
-        // Check if the standard tag exists
-        Exiv2::ExifKey key("Exif.Photo.ISOSpeedRatings");
-        Exiv2::ExifData::iterator md = exifData.findKey(key);
+        Exiv2::ExifData::const_iterator md = Exiv2::isoSpeed(exifData);
         if (md != exifData.end()) {
-            if (Params::instance().verbose_) {
-                std::cout << _("Standard Exif ISO tag exists; not modified\n");
+            if (strcmp(md->key().c_str(), "Exif.Photo.ISOSpeedRatings") == 0) {
+                if (Params::instance().verbose_) {
+                    std::cout << _("Standard Exif ISO tag exists; not modified\n");
+                }
+                return 0;
             }
-            return 0;
-        }
-        // Fix Nikon ISO setting
-        key = Exiv2::ExifKey("Exif.Nikon3.ISOSpeed");
-        md = exifData.findKey(key);
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.Nikon2.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.Nikon1.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        // Canon has a similar bad habit, fix that too
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.CanonSi.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        if (md == exifData.end()) {
-            key = Exiv2::ExifKey("Exif.CanonCs.ISOSpeed");
-            md = exifData.findKey(key);
-        }
-        // Copy the proprietary tag to the standard place
-        if (md != exifData.end()) {
+            // Copy the proprietary tag to the standard place
             std::ostringstream os;
             md->write(os, &exifData);
             if (Params::instance().verbose_) {
@@ -1591,6 +1640,84 @@ namespace Action {
         return new FixIso(*this);
     }
 
+    FixCom::~FixCom()
+    {
+    }
+
+    int FixCom::run(const std::string& path)
+    {
+    try {
+        if (!Exiv2::fileExists(path, true)) {
+            std::cerr << path
+                      << ": " <<_("Failed to open the file\n");
+            return -1;
+        }
+        Timestamp ts;
+        if (Params::instance().preserve_) {
+            ts.read(path);
+        }
+        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path);
+        assert(image.get() != 0);
+        image->readMetadata();
+        Exiv2::ExifData& exifData = image->exifData();
+        if (exifData.empty()) {
+            std::cerr << path
+                      << ": " << _("No Exif data found in the file\n");
+            return -3;
+        }
+        Exiv2::ExifData::iterator pos = exifData.findKey(Exiv2::ExifKey("Exif.Photo.UserComment"));
+        if (pos == exifData.end()) {
+            if (Params::instance().verbose_) {
+                std::cout << _("No Exif user comment found") << "\n";
+            }
+            return 0;
+        }
+        Exiv2::Value::AutoPtr v = pos->getValue();
+        const Exiv2::CommentValue* pcv = dynamic_cast<const Exiv2::CommentValue*>(v.get());
+        if (!pcv) {
+            if (Params::instance().verbose_) {
+                std::cout << _("Found Exif user comment with unexpected value type") << "\n";
+            }
+            return 0;
+        }
+        Exiv2::CommentValue::CharsetId csId = pcv->charsetId();
+        if (csId != Exiv2::CommentValue::unicode) {
+            if (Params::instance().verbose_) {
+                std::cout << _("No Exif UNICODE user comment found") << "\n";
+            }
+            return 0;
+        }
+        std::string comment = pcv->comment(Params::instance().charset_.c_str());
+        if (Params::instance().verbose_) {
+            std::cout << _("Setting Exif UNICODE user comment to") << " \"" << comment << "\"\n";
+        }
+        comment = std::string("charset=\"") + Exiv2::CommentValue::CharsetInfo::name(csId) + "\" " + comment;
+        // Remove BOM and convert value from source charset to UCS-2, but keep byte order
+        pos->setValue(comment);
+        image->writeMetadata();
+        if (Params::instance().preserve_) {
+            ts.touch(path);
+        }
+        return 0;
+    }
+    catch(const Exiv2::AnyError& e)
+    {
+        std::cerr << "Exiv2 exception in fixcom action for file " << path
+                  << ":\n" << e << "\n";
+        return 1;
+    }
+    } // FixCom::run
+
+    FixCom::AutoPtr FixCom::clone() const
+    {
+        return AutoPtr(clone_());
+    }
+
+    FixCom* FixCom::clone_() const
+    {
+        return new FixCom(*this);
+    }
+
 }                                       // namespace Action
 
 // *****************************************************************************
@@ -1655,6 +1782,13 @@ namespace {
         if (!Util::strtol(timeStr.substr(17,2).c_str(), tmp)) return 10;
         tm->tm_sec = tmp;
 
+        // Conversions to set remaining fields of the tm structure
+        time_t time = timegm(tm);
+#ifdef EXV_HAVE_GMTIME_R
+        if (time == (time_t)-1 || gmtime_r(&time, tm) == 0) return 11;
+#else
+        if (time == (time_t)-1 || std::gmtime(&time)  == 0) return 11;
+#endif
         return 0;
     } // str2Tm
 
@@ -1713,22 +1847,7 @@ namespace {
                 std::cout << _("Writing Exif data from") << " " << source
                           << " " << _("to") << " " << target << std::endl;
             }
-            if (0 == strcmp(targetImage->mimeType().c_str(), "image/tiff")) {
-                Exiv2::ExifData& ted = targetImage->exifData();
-                if (!preserve) {
-                    targetImage->readMetadata();
-                    ted.erase(std::remove_if(ted.begin(), ted.end(), isExifTag), ted.end());
-                }
-                const Exiv2::ExifData& sed = sourceImage->exifData();
-                for (Exiv2::ExifData::const_iterator pos = sed.begin(); pos != sed.end(); ++pos) {
-                    if (isExifTag(*pos)) {
-                        ted[pos->key()] = pos->value();
-                    }
-                }
-            }
-            else {
-                targetImage->setExifData(sourceImage->exifData());
-            }
+            targetImage->setExifData(sourceImage->exifData());
         }
         if (   Params::instance().target_ & Params::ctIptc
             && !sourceImage->iptcData().empty()) {
@@ -1774,66 +1893,6 @@ namespace {
         }
     };
 
-    bool isExifTag(const Exiv2::Exifdatum& ed)
-    {
-        // A somewhat random list of IFD0 tags which are considered as "Exif tags"
-        static const String exifTags[] = {
-            { "Exif.Image.ProcessingSoftware"      },
-            { "Exif.Image.DocumentName"            },
-            { "Exif.Image.ImageDescription"        },
-            { "Exif.Image.Make"                    },
-            { "Exif.Image.Model"                   },
-            { "Exif.Image.Software"                },
-            { "Exif.Image.DateTime"                },
-            { "Exif.Image.HostComputer"            },
-            { "Exif.Image.Artist"                  },
-            { "Exif.Image.XMLPacket"               },
-            { "Exif.Image.Rating"                  },
-            { "Exif.Image.RatingPercent"           },
-            { "Exif.Image.CFARepeatPatternDim"     },
-            { "Exif.Image.CFAPattern"              },
-            { "Exif.Image.BatteryLevel"            },
-            { "Exif.Image.IPTCNAA"                 },
-            { "Exif.Image.Copyright"               },
-            { "Exif.Image.ImageResources"          },
-            { "Exif.Image.ExifTag"                 },
-            { "Exif.Image.InterColorProfile"       },
-            { "Exif.Image.GPSTag"                  },
-            { "Exif.Image.XPTitle"                 },
-            { "Exif.Image.XPComment"               },
-            { "Exif.Image.XPAuthor"                },
-            { "Exif.Image.XPKeywords"              },
-            { "Exif.Image.XPSubject"               },
-            { "Exif.Image.PrintImageMatching"      },
-            { "Exif.Image.UniqueCameraModel"       },
-            { "Exif.Image.LocalizedCameraModel"    },
-            { "Exif.Image.CFAPlaneColor"           },
-            { "Exif.Image.CFALayout"               },
-            { "Exif.Image.CameraSerialNumber"      },
-            { "Exif.Image.LensInfo"                },
-            { "Exif.Image.OriginalRawFileName"     },
-            { "Exif.Image.ActiveArea"              },
-            { "Exif.Image.MaskedAreas"             },
-            { "Exif.Image.AsShotICCProfile"        },
-            { "Exif.Image.AsShotPreProfileMatrix"  },
-            { "Exif.Image.CurrentICCProfile"       },
-            { "Exif.Image.CurrentPreProfileMatrix" }
-        };
-
-        static const Exiv2::IfdId exifIfds[] = {
-            Exiv2::exifIfdId,
-            Exiv2::gpsIfdId,
-            Exiv2::iopIfdId
-        };
-
-        if (   0 != Exiv2::find(exifIfds, ed.ifdId())
-            || Exiv2::ExifTags::isMakerIfd(ed.ifdId())
-            || 0 != Exiv2::find(exifTags, ed.key().c_str())) {
-            return true;
-        }
-        return false;
-    } // isExifTag
-
     int renameFile(std::string& newPath, const struct tm* tm)
     {
         std::string path = newPath;
