@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,15 +20,15 @@
  */
 /*
   File:      convert.cpp
-  Version:   $Rev: 1750 $
+  Version:   $Rev: 2455 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Vladimir Nadvornik (vn) <nadvornik@suse.cz>
   History:   17-Mar-08, ahu: created basic converter framework
              20-May-08, vn:  added actual conversion logic
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: convert.cpp 1750 2009-02-16 14:30:51Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: convert.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -36,16 +36,24 @@ EXIV2_RCSID("@(#) $Id: convert.cpp 1750 2009-02-16 14:30:51Z ahuggel $")
 #include "exif.hpp"
 #include "iptc.hpp"
 #include "xmp.hpp"
+#include "futils.hpp"
 #include "convert.hpp"
 
 // + standard includes
 #include <iostream>
 #include <iomanip>
+#include <ios>
 #include <sstream>
 #include <stdio.h> // for snprintf (C99)
 #ifdef _MSC_VER
 # define snprintf _snprintf
 #endif
+#include <cstring>
+
+#ifdef EXV_HAVE_ICONV
+# include <iconv.h>
+# include <errno.h>
+#endif
 
 // Adobe XMP Toolkit
 #ifdef EXV_HAVE_XMP_TOOLKIT
@@ -97,7 +105,7 @@ namespace Exiv2 {
         //! Constructor for Exif tags and XMP properties.
         Converter(ExifData& exifData, XmpData& xmpData);
         //! Constructor for Iptc tags and XMP properties.
-        Converter(IptcData& iptcData, XmpData& xmpData);
+        Converter(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset = 0);
         //@}
 
         //! @name Manipulators
@@ -277,6 +285,7 @@ namespace Exiv2 {
         ExifData *exifData_;
         IptcData *iptcData_;
         XmpData  *xmpData_;
+        const char *iptcCharset_;
 
     }; // class Converter
 
@@ -288,7 +297,7 @@ namespace Exiv2 {
         { mdExif, "Exif.Image.Compression",               "Xmp.tiff.Compression",               &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.PhotometricInterpretation", "Xmp.tiff.PhotometricInterpretation", &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Orientation",               "Xmp.tiff.Orientation",               &Converter::cnvExifValue, &Converter::cnvXmpValue },
-        { mdExif, "Exif.Image.SamplesPerPixel",           "Xmp.tiff.SamplesPerPixe",            &Converter::cnvExifValue, &Converter::cnvXmpValue },
+        { mdExif, "Exif.Image.SamplesPerPixel",           "Xmp.tiff.SamplesPerPixel",           &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.PlanarConfiguration",       "Xmp.tiff.PlanarConfiguration",       &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.YCbCrSubSampling",          "Xmp.tiff.YCbCrSubSampling",          &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.YCbCrPositioning",          "Xmp.tiff.YCbCrPositioning",          &Converter::cnvExifValue, &Converter::cnvXmpValue },
@@ -391,8 +400,8 @@ namespace Exiv2 {
         { mdIptc, "Iptc.Application2.Category",           "Xmp.photoshop.Category",             &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.SuppCategory",       "Xmp.photoshop.SupplementalCategory", &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.Keywords",           "Xmp.dc.subject",                     &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
-        { mdIptc, "Iptc.Application2.LocationName",       "Xmp.iptc.Location",                  &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
-        { mdIptc, "Iptc.Application2.SpecialInstructions","Xmp.photoshop.Instruction",          &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
+        { mdIptc, "Iptc.Application2.SubLocation",        "Xmp.iptc.Location",                  &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
+        { mdIptc, "Iptc.Application2.SpecialInstructions","Xmp.photoshop.Instructions",         &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.DateCreated",        "Xmp.photoshop.DateCreated",          &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.Byline",             "Xmp.dc.creator",                     &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.BylineTitle",        "Xmp.photoshop.AuthorsPosition",      &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
@@ -411,12 +420,12 @@ namespace Exiv2 {
     };
 
     Converter::Converter(ExifData& exifData, XmpData& xmpData)
-        : erase_(false), overwrite_(true), exifData_(&exifData), iptcData_(0), xmpData_(&xmpData)
+        : erase_(false), overwrite_(true), exifData_(&exifData), iptcData_(0), xmpData_(&xmpData), iptcCharset_(0)
     {
     }
 
-    Converter::Converter(IptcData& iptcData, XmpData& xmpData)
-        : erase_(false), overwrite_(true), exifData_(0), iptcData_(&iptcData), xmpData_(&xmpData)
+    Converter::Converter(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset)
+        : erase_(false), overwrite_(true), exifData_(0), iptcData_(&iptcData), xmpData_(&xmpData), iptcCharset_(iptcCharset)
     {
     }
 
@@ -478,7 +487,7 @@ namespace Exiv2 {
         std::string value = pos->toString();
         if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -495,7 +504,7 @@ namespace Exiv2 {
         const CommentValue* cv = dynamic_cast<const CommentValue*>(&pos->value());
         if (cv == 0) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -513,7 +522,7 @@ namespace Exiv2 {
             std::string value = pos->toString(i);
             if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
@@ -535,14 +544,14 @@ namespace Exiv2 {
             std::string value = pos->toString();
             if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
             if (sscanf(value.c_str(), "%d:%d:%d %d:%d:%d", &year, &month, &day, &hour, &min, &sec) != 6) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to
-                          << ", unable to parse '" << value << "'\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to
+                            << ", unable to parse '" << value << "'\n";
 #endif
                 return;
             }
@@ -561,7 +570,7 @@ namespace Exiv2 {
             }
             if (!ok) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
@@ -574,7 +583,7 @@ namespace Exiv2 {
 
             if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
@@ -590,6 +599,7 @@ namespace Exiv2 {
 
             snprintf(buf, sizeof(buf), "%.9f", dsec);
             buf[sizeof(buf) - 1] = 0;
+            buf[1] = '.'; // some locales use ','
             subsec = buf + 1;
 
             Exiv2::ExifData::iterator datePos = exifData_->findKey(ExifKey("Exif.GPSInfo.GPSDateStamp"));
@@ -601,15 +611,15 @@ namespace Exiv2 {
             }
             if (datePos == exifData_->end()) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
             std::string value = datePos->toString();
             if (sscanf(value.c_str(), "%d:%d:%d", &year, &month, &day) != 3) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to
-                          << ", unable to parse '" << value << "'\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to
+                            << ", unable to parse '" << value << "'\n";
 #endif
                 return;
             }
@@ -627,10 +637,15 @@ namespace Exiv2 {
         }
 
         if (subsecTag) {
-            Exiv2::ExifData::iterator subsec_pos = exifData_->findKey(ExifKey(subsecTag));
+            ExifData::iterator subsec_pos = exifData_->findKey(ExifKey(subsecTag));
             if (   subsec_pos != exifData_->end()
-                && !subsec_pos->toString().empty()) {
-                subsec = std::string(".") + subsec_pos->toString();
+                && subsec_pos->typeId() == asciiString) {
+                std::string ss = subsec_pos->toString();
+                if (!ss.empty()) {
+                    bool ok = false;
+                    stringTo<long>(ss, ok);
+                    if (ok) subsec = std::string(".") + ss;
+                }
             }
             if (erase_) exifData_->erase(subsec_pos);
         }
@@ -679,7 +694,7 @@ namespace Exiv2 {
         int value = pos->toLong();
         if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -700,14 +715,14 @@ namespace Exiv2 {
         if (!prepareXmpTarget(to)) return;
         if (pos->count() != 3) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
         Exiv2::ExifData::iterator refPos = exifData_->findKey(ExifKey(std::string(from) + "Ref"));
         if (refPos == exifData_->end()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -717,7 +732,7 @@ namespace Exiv2 {
             const int32_t d = pos->toRational(i).second;
             if (d == 0) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
@@ -745,7 +760,7 @@ namespace Exiv2 {
         std::string value;
         if (!getTextValue(value, pos)) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -766,7 +781,7 @@ namespace Exiv2 {
         std::string value;
         if (!getTextValue(value, pos)) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -785,11 +800,12 @@ namespace Exiv2 {
             std::string value = pos->toString(i);
             if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
-            array << value << " ";
+            array << value;
+            if (i != pos->count() - 1) array << " ";
         }
         (*exifData_)[to] = array.str();
         if (erase_) xmpData_->erase(pos);
@@ -804,12 +820,24 @@ namespace Exiv2 {
         std::string value = pos->toString();
         if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
         XMP_DateTime datetime;
-        SXMPUtils::ConvertToDate(value, &datetime);
+        try {
+            SXMPUtils::ConvertToDate(value, &datetime);
+        }
+#ifndef SUPPRESS_WARNINGS
+        catch (const XMP_Error& e) {
+            EXV_WARNING << "Failed to convert " << from << " to " << to << " (" << e.GetErrMsg() << ")\n";
+            return;
+        }
+#else
+        catch (const XMP_Error&) {
+            return;
+        }
+#endif // SUPPRESS_WARNINGS
         char buf[30];
         if (std::string(to) != "Exif.GPSInfo.GPSTimeStamp") {
 
@@ -876,7 +904,7 @@ namespace Exiv2 {
         if (erase_) xmpData_->erase(pos);
 #else
 # ifndef SUPPRESS_WARNINGS
-        std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+        EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 # endif
 #endif // !EXV_HAVE_XMP_TOOLKIT
     }
@@ -889,7 +917,7 @@ namespace Exiv2 {
         std::string value = pos->toString();
         if (!pos->value().ok() || value.length() < 4) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -912,7 +940,7 @@ namespace Exiv2 {
         std::string value = pos->toString();
         if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -938,7 +966,7 @@ namespace Exiv2 {
                 value |= fired & 1;
 #ifndef SUPPRESS_WARNINGS
             else
-                std::cerr << "Warning: Failed to convert " << std::string(from) + "/exif:Fired" << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << std::string(from) + "/exif:Fired" << " to " << to << "\n";
 #endif
         }
         pos = xmpData_->findKey(XmpKey(std::string(from) + "/exif:Return"));
@@ -948,7 +976,7 @@ namespace Exiv2 {
                 value |= (ret & 3) << 1;
 #ifndef SUPPRESS_WARNINGS
             else
-                std::cerr << "Warning: Failed to convert " << std::string(from) + "/exif:Return" << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << std::string(from) + "/exif:Return" << " to " << to << "\n";
 #endif
         }
         pos = xmpData_->findKey(XmpKey(std::string(from) + "/exif:Mode"));
@@ -958,7 +986,7 @@ namespace Exiv2 {
                 value |= (mode & 3) << 3;
 #ifndef SUPPRESS_WARNINGS
             else
-                std::cerr << "Warning: Failed to convert " << std::string(from) + "/exif:Mode" << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << std::string(from) + "/exif:Mode" << " to " << to << "\n";
 #endif
         }
         pos = xmpData_->findKey(XmpKey(std::string(from) + "/exif:Function"));
@@ -968,7 +996,7 @@ namespace Exiv2 {
                 value |= (function & 1) << 5;
 #ifndef SUPPRESS_WARNINGS
             else
-                std::cerr << "Warning: Failed to convert " << std::string(from) + "/exif:Function" << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << std::string(from) + "/exif:Function" << " to " << to << "\n";
 #endif
         }
         pos = xmpData_->findKey(XmpKey(std::string(from) + "/exif:RedEyeMode"));
@@ -978,7 +1006,7 @@ namespace Exiv2 {
                 value |= (red & 1) << 6;
 #ifndef SUPPRESS_WARNINGS
             else
-                std::cerr << "Warning: Failed to convert " << std::string(from) + "/exif:RedEyeMode" << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << std::string(from) + "/exif:RedEyeMode" << " to " << to << "\n";
 #endif
         }
 
@@ -994,13 +1022,23 @@ namespace Exiv2 {
         std::string value = pos->toString();
         if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
+#endif
+            return;
+        }
+        if (value.empty()) {
+#ifndef SUPPRESS_WARNINGS
+            EXV_WARNING << from << " is empty\n";
 #endif
             return;
         }
 
-        double deg, min, sec;
-        char ref, sep1, sep2;
+        double deg = 0.0;
+        double min = 0.0;
+        double sec = 0.0;
+        char ref  = '\0';
+        char sep1 = '\0';
+        char sep2 = '\0';
 
         ref = value[value.length() - 1];
         value.erase(value.length() - 1);
@@ -1021,7 +1059,7 @@ namespace Exiv2 {
         if (   in.bad() || !(ref == 'N' || ref == 'S' || ref == 'E' || ref == 'W')
             || sep1 != ',' || sep2 != ',' || !in.eof()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+            EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
             return;
         }
@@ -1051,11 +1089,12 @@ namespace Exiv2 {
                 std::string value = pos->toString();
                 if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                    EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                     ++pos;
                     continue;
                 }
+                if (iptcCharset_) convertStringCharset(value, iptcCharset_, "UTF-8");
                 (*xmpData_)[to] = value;
                 if (erase_) {
                     pos = iptcData_->erase(pos);
@@ -1076,31 +1115,33 @@ namespace Exiv2 {
             std::string value;
             if (!getTextValue(value, pos)) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 return;
             }
-            // Todo: Escape non-ASCII characters in XMP text values
             (*iptcData_)[to] = value;
+            (*iptcData_)["Iptc.Envelope.CharacterSet"] = "\033%G"; // indicate UTF-8 encoding
             if (erase_) xmpData_->erase(pos);
             return;
         }
 
         int count = pos->count();
+        bool added = false;
         for (int i = 0; i < count; ++i) {
             std::string value = pos->toString(i);
             if (!pos->value().ok()) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+                EXV_WARNING << "Failed to convert " << from << " to " << to << "\n";
 #endif
                 continue;
             }
-            // Todo: Escape non-ASCII characters in XMP text values
             IptcKey key(to);
             Iptcdatum id(key);
             id.setValue(value);
             iptcData_->add(id);
+            added = true;
         }
+        if (added) (*iptcData_)["Iptc.Envelope.CharacterSet"] = "\033%G"; // indicate UTF-8 encoding
         if (erase_) xmpData_->erase(pos);
     }
 
@@ -1208,6 +1249,7 @@ namespace Exiv2 {
 #endif
     }
 
+
     // *************************************************************************
     // free functions
     void copyExifToXmp(const ExifData& exifData, XmpData& xmpData)
@@ -1242,15 +1284,20 @@ namespace Exiv2 {
         converter.syncExifWithXmp();
     }
 
-    void copyIptcToXmp(const IptcData& iptcData, XmpData& xmpData)
+    void copyIptcToXmp(const IptcData& iptcData, XmpData& xmpData, const char *iptcCharset)
     {
-        Converter converter(const_cast<IptcData&>(iptcData), xmpData);
+        if (!iptcCharset) iptcCharset = iptcData.detectCharset();
+        if (!iptcCharset) iptcCharset = "ISO-8859-1";
+
+        Converter converter(const_cast<IptcData&>(iptcData), xmpData, iptcCharset);
         converter.cnvToXmp();
     }
 
-    void moveIptcToXmp(IptcData& iptcData, XmpData& xmpData)
+    void moveIptcToXmp(IptcData& iptcData, XmpData& xmpData, const char *iptcCharset)
     {
-        Converter converter(const_cast<IptcData&>(iptcData), xmpData);
+        if (!iptcCharset) iptcCharset = iptcData.detectCharset();
+        if (!iptcCharset) iptcCharset = "ISO-8859-1";
+        Converter converter(const_cast<IptcData&>(iptcData), xmpData, iptcCharset);
         converter.setErase();
         converter.cnvToXmp();
     }
@@ -1268,6 +1315,54 @@ namespace Exiv2 {
         converter.cnvFromXmp();
     }
 
+    bool convertStringCharset(std::string &str, const char* from, const char* to)
+    {
+        if (0 == strcmp(from, to)) return true; // nothing to do
+#if defined EXV_HAVE_ICONV
+        bool ret = true;
+        iconv_t cd;
+        cd = iconv_open(to, from);
+        if (cd == (iconv_t)(-1)) {
+#ifndef SUPPRESS_WARNINGS
+            EXV_WARNING << "iconv_open: " << strError() << "\n";
+#endif
+            return false;
+        }
+        std::string outstr;
+        EXV_ICONV_CONST char *inptr = const_cast<char *>(str.c_str());
+        size_t inbytesleft = str.length();
+        while (inbytesleft) {
+            char outbuf[100];
+            char *outptr = outbuf;
+            size_t outbytesleft = sizeof(outbuf) - 1;
+            size_t rc = iconv(cd,
+                              &inptr,
+                              &inbytesleft,
+                              &outptr,
+                              &outbytesleft);
+            int outbytesProduced = sizeof(outbuf) - 1 - outbytesleft;
+            if (rc == size_t(-1) && errno != E2BIG) {
+#ifndef SUPPRESS_WARNINGS
+                EXV_WARNING << "iconv: " << strError()
+                            << " inbytesleft = " << inbytesleft << "\n";
+#endif
+                ret = false;
+                break;
+            }
+            *outptr = '\0';
+            outstr.append(std::string(outbuf, outbytesProduced));
+        }
+        if (cd != (iconv_t)(-1)) {
+            iconv_close(cd);
+        }
+
+        if (ret) str = outstr;
+        return ret;
+#else // !EXV_HAVE_ICONV
+        return false;
+#endif // EXV_HAVE_ICONV
+    }
+
 }                                       // namespace Exiv2
 
 // *****************************************************************************
