@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    crwimage_int.hpp
   @brief   Internal classes to support CRW/CIFF format.
-  @version $Rev: 1750 $
+  @version $Rev: 2455 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    28-Aug-05, ahu: created
@@ -32,6 +32,7 @@
 // *****************************************************************************
 // included header files
 #include "types.hpp"
+#include "tags_int.hpp"
 #include "image.hpp"
 #include "basicio.hpp"
 
@@ -322,7 +323,7 @@ namespace Exiv2 {
         CiffEntry(uint16_t tag, uint16_t dir) : CiffComponent(tag, dir) {}
 
         //! Virtual destructor.
-        virtual ~CiffEntry() {}
+        virtual ~CiffEntry();
         //@}
 
         // Default assignment operator is fine
@@ -556,7 +557,7 @@ namespace Exiv2 {
             uint16_t      crwDir,
             uint32_t      size,
             uint16_t      tag,
-            IfdId         ifdId,
+            Internal::IfdId ifdId,
             CrwDecodeFct  toExif,
             CrwEncodeFct  fromExif)
             : crwTagId_ (crwTagId),
