@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,9 +20,11 @@
  */
 /*!
   @file    makernote_int.hpp
-  @brief   Internal Makernote TIFF composite class TiffIfdMakernote and classes
-           for various makernote headers.
-  @version $Rev: 1803 $
+  @brief   Makernote factory and registry, IFD makernote header, and camera
+           vendor specific makernote implementations.<BR>References:<BR>
+  [1] <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/">ExifTool</a> by Phil Harvey<BR>
+  [2] <a href="http://www.cybercom.net/~dcoffin/dcraw/">Decoding raw digital photos in Linux</a> by Dave Coffin
+  @version $Rev: 2455 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Apr-06, ahu: created
@@ -32,7 +34,8 @@
 
 // *****************************************************************************
 // included header files
-#include "tiffcomposite_int.hpp"
+#include "tifffwd_int.hpp"
+#include "tags_int.hpp"
 #include "types.hpp"
 
 // + standard includes
@@ -43,66 +46,21 @@
 namespace Exiv2 {
     namespace Internal {
 
-    namespace Group {
-        const uint16_t olympmn   = 257; //!< any Olympus makernote
-        const uint16_t fujimn    = 258; //!< Fujifilm makernote
-        const uint16_t canonmn   = 259; //!< Canon makernote
-        const uint16_t canoncs   = 260; //!< Canon camera settings
-        const uint16_t canonsi   = 261; //!< Canon shot info
-        const uint16_t canoncf   = 262; //!< Canon custom functions
-        const uint16_t nikonmn   = 263; //!< Any Nikon makernote (pseudo group)
-        const uint16_t nikon1mn  = 264; //!< Nikon1 makernote
-        const uint16_t nikon2mn  = 265; //!< Nikon2 makernote
-        const uint16_t nikon3mn  = 266; //!< Nikon3 makernote
-        const uint16_t panamn    = 267; //!< Panasonic makernote
-        const uint16_t sigmamn   = 268; //!< Sigma makernote
-        const uint16_t sonymn    = 269; //!< Any Sony makernote (pseudo group)
-        const uint16_t sony1mn   = 270; //!< Sony1 makernote
-        const uint16_t sony2mn   = 271; //!< Sony2 makernote
-        const uint16_t minoltamn = 272; //!< Minolta makernote
-        const uint16_t minocso   = 273; //!< Minolta camera settings (old)
-        const uint16_t minocsn   = 274; //!< Minolta camera settings (new)
-        const uint16_t minocs5   = 275; //!< Minolta camera settings (D5)
-        const uint16_t minocs7   = 276; //!< Minolta camera settings (D7)
-        const uint16_t canonpi   = 277; //!< Canon picture info
-        const uint16_t canonpa   = 278; //!< Canon panorama
-        const uint16_t pentaxmn  = 279; //!< Pentax makernote
-        const uint16_t nikonpv   = 280; //!< Nikon preview sub-IFD
-        const uint16_t olymp1mn  = 281; //!< Olympus makernote
-        const uint16_t olymp2mn  = 282; //!< Olympus II makernote
-        const uint16_t olympcs   = 283; //!< Olympus camera settings
-        const uint16_t olympeq   = 284; //!< Olympus equipment tags
-        const uint16_t olymprd   = 285; //!< Olympus raw development tags
-        const uint16_t olymprd2  = 286; //!< Olympus raw development 2 tags
-        const uint16_t olympip   = 287; //!< Olympus image processing tags
-        const uint16_t olympfi   = 288; //!< Olympus focus info tags
-        const uint16_t olympfe1  = 289; //!< Olympus FE 1 tags
-        const uint16_t olympfe2  = 290; //!< Olympus FE 2 tags
-        const uint16_t olympfe3  = 291; //!< Olympus FE 3 tags
-        const uint16_t olympfe4  = 292; //!< Olympus FE 4 tags
-        const uint16_t olympfe5  = 293; //!< Olympus FE 5 tags
-        const uint16_t olympfe6  = 294; //!< Olympus FE 6 tags
-        const uint16_t olympfe7  = 295; //!< Olympus FE 7 tags
-        const uint16_t olympfe8  = 296; //!< Olympus FE 8 tags
-        const uint16_t olympfe9  = 297; //!< Olympus FE 9 tags
-        const uint16_t olympri   = 298; //!< Olympus raw info tags
-    }
-
 // *****************************************************************************
 // class definitions
 
     //! Type for a pointer to a function creating a makernote (image)
     typedef TiffComponent* (*NewMnFct)(uint16_t    tag,
-                                       uint16_t    group,
-                                       uint16_t    mnGroup,
+                                       IfdId       group,
+                                       IfdId       mnGroup,
                                        const byte* pData,
                                        uint32_t    size,
                                        ByteOrder   byteOrder);
 
     //! Type for a pointer to a function creating a makernote (group)
     typedef TiffComponent* (*NewMnFct2)(uint16_t   tag,
-                                        uint16_t   group,
-                                        uint16_t   mnGroup);
+                                        IfdId      group,
+                                        IfdId      mnGroup);
 
     //! Makernote registry structure
     struct TiffMnRegistry {
@@ -117,11 +75,11 @@ namespace Exiv2 {
         bool operator==(const std::string& key) const;
 
         //! Compare a TiffMnRegistry structure with a makernote group
-        bool operator==(const uint16_t& key) const;
+        bool operator==(IfdId key) const;
 
         // DATA
         const char* make_;                      //!< Camera make
-        uint16_t    mnGroup_;                   //!< Group identifier
+        IfdId       mnGroup_;                   //!< Group identifier
         NewMnFct    newMnFct_;                  //!< Makernote create function (image)
         NewMnFct2   newMnFct2_;                 //!< Makernote create function (group)
     };
@@ -143,7 +101,7 @@ namespace Exiv2 {
                  file dependencies.
         */
         static TiffComponent* create(uint16_t           tag,
-                                     uint16_t           group,
+                                     IfdId              group,
                                      const std::string& make,
                                      const byte*        pData,
                                      uint32_t           size,
@@ -153,8 +111,8 @@ namespace Exiv2 {
                  when a makernote is written back from Exif tags.
          */
         static TiffComponent* create(uint16_t           tag,
-                                     uint16_t           group,
-                                     uint16_t           mnGroup);
+                                     IfdId              group,
+                                     IfdId              mnGroup);
 
     protected:
         //! Prevent destruction (needed if used as a policy class)
@@ -169,7 +127,7 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         //! Virtual destructor.
-        virtual ~MnHeader() {}
+        virtual ~MnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -187,179 +145,29 @@ namespace Exiv2 {
         //! Return the size of the header (in bytes).
         virtual uint32_t size() const =0;
         //! Write the header to a data buffer, return the number of bytes written.
-        virtual uint32_t write(Blob&     blob,
-                               ByteOrder byteOrder) const =0;
+        virtual uint32_t write(IoWrapper& ioWrapper,
+                               ByteOrder  byteOrder) const =0;
         /*!
           @brief Return the offset to the start of the Makernote IFD from
                  the start of the Makernote (= the start of the header).
          */
-        virtual uint32_t ifdOffset() const { return 0; }
+        virtual uint32_t ifdOffset() const;
         /*!
           @brief Return the byte order for the makernote. If the return value is
                  invalidByteOrder, this means that the byte order of the the
                  image should be used for the makernote.
          */
-        virtual ByteOrder byteOrder() const { return invalidByteOrder; }
+        virtual ByteOrder byteOrder() const;
         /*!
           @brief Return the base offset for the makernote IFD entries relative
                  to the start of the TIFF header. \em mnOffset is the offset
                  to the makernote from the start of the TIFF header.
          */
-        virtual uint32_t baseOffset(uint32_t /*mnOffset*/) const { return 0; }
+        virtual uint32_t baseOffset(uint32_t mnOffset) const;
         //@}
 
     }; // class MnHeader
 
-    /*!
-      @brief Tiff IFD Makernote. This is a concrete class suitable for all
-             IFD makernotes.
-
-             Contains a makernote header (which can be 0) and an IFD and
-             implements child mgmt functions to deal with the IFD entries. The
-             various makernote weirdnesses are taken care of in the makernote
-             header (and possibly in special purpose IFD entries).
-     */
-    class TiffIfdMakernote : public TiffComponent {
-        friend class TiffReader;
-    public:
-        //! @name Creators
-        //@{
-        //! Default constructor
-        TiffIfdMakernote(uint16_t  tag,
-                         uint16_t  group,
-                         uint16_t  mnGroup,
-                         MnHeader* pHeader,
-                         bool      hasNext =true);
-        //! Virtual destructor
-        virtual ~TiffIfdMakernote();
-        //@}
-
-        //! @name Manipulators
-        //@{
-        /*!
-          @brief Read the header from a data buffer, return true if successful.
-
-          The default implementation simply returns true.
-         */
-        bool readHeader(const byte* pData, uint32_t size, ByteOrder byteOrder);
-        /*!
-          @brief Set the byte order for the makernote.
-         */
-        void setByteOrder(ByteOrder byteOrder);
-        /*!
-          @brief Set the byte order used for the image.
-         */
-        void setImageByteOrder(ByteOrder byteOrder) { imageByteOrder_ = byteOrder; }
-        //@}
-
-        //! @name Accessors
-        //@{
-        //! Return the size of the header in bytes.
-        uint32_t sizeHeader() const;
-        //! Write the header to a data buffer, return the number of bytes written.
-        uint32_t writeHeader(Blob& blob, ByteOrder byteOrder) const;
-        /*!
-          @brief Return the offset to the makernote from the start of the
-                 TIFF header.
-        */
-        uint32_t mnOffset() const;
-        /*!
-          @brief Return the offset to the start of the Makernote IFD from
-                 the start of the Makernote.
-                 Returns 0 if there is no header.
-         */
-        uint32_t ifdOffset() const;
-        /*!
-          @brief Return the byte order for the makernote. Requires the image
-                 byte order to be set (setImageByteOrder()).  Returns the byte
-                 order for the image if there is no header or the byte order for
-                 the header is \c invalidByteOrder.
-         */
-        ByteOrder byteOrder() const;
-        /*!
-          @brief Return the byte order used for the image.
-         */
-        ByteOrder imageByteOrder() const { return imageByteOrder_; }
-        /*!
-          @brief Return the base offset for use with the makernote IFD entries
-                 relative to the start of the TIFF header.
-                 Returns 0 if there is no header.
-         */
-        uint32_t baseOffset() const;
-        //@}
-
-    protected:
-        //! @name Manipulators
-        //@{
-        virtual TiffComponent* doAddPath(uint16_t tag, TiffPath& tiffPath);
-        virtual TiffComponent* doAddChild(TiffComponent::AutoPtr tiffComponent);
-        virtual TiffComponent* doAddNext(TiffComponent::AutoPtr tiffComponent);
-        virtual void doAccept(TiffVisitor& visitor);
-        //@}
-
-        //! @name Write support (Manipulators)
-        //@{
-        /*!
-          @brief Implements write(). Write the Makernote header, TIFF directory,
-                 values and additional data to the blob, return the number of
-                 bytes written.
-         */
-        virtual uint32_t doWrite(Blob&     blob,
-                                 ByteOrder byteOrder,
-                                 int32_t   offset,
-                                 uint32_t  valueIdx,
-                                 uint32_t  dataIdx,
-                                 uint32_t& imageIdx);
-        //@}
-        //! @name Write support (Accessors)
-        //@{
-        /*!
-          @brief This class does not really implement writeData(), it only has
-                 write(). This method must not be called; it commits suicide.
-         */
-        virtual uint32_t doWriteData(Blob&     blob,
-                                     ByteOrder byteOrder,
-                                     int32_t   offset,
-                                     uint32_t  dataIdx,
-                                     uint32_t& imageIdx) const;
-        /*!
-          @brief Implements writeImage(). Write the image data of the IFD of
-                 the Makernote. Return the number of bytes written.
-         */
-        virtual uint32_t doWriteImage(Blob&     blob,
-                                      ByteOrder byteOrder) const;
-        /*!
-          @brief Implements size(). Return the size of the Makernote header,
-                 TIFF directory, values and additional data.
-         */
-        virtual uint32_t doSize() const;
-        /*!
-          @brief Implements count(). Return the number of entries in the IFD
-                 of the Makernote. Does not count entries which are marked as
-                 deleted.
-         */
-        virtual uint32_t doCount() const;
-        /*!
-          @brief This class does not really implement sizeData(), it only has
-                 size(). This method must not be called; it commits suicide.
-         */
-        virtual uint32_t doSizeData() const;
-        /*!
-          @brief Implements sizeImage(). Return the total image data size of the
-                 makernote IFD.
-         */
-        virtual uint32_t doSizeImage() const;
-        //@}
-
-    private:
-        // DATA
-        MnHeader*     pHeader_;                 //!< Makernote header
-        TiffDirectory ifd_;                     //!< Makernote IFD
-        uint32_t      mnOffset_;                //!< Makernote offset
-        ByteOrder     imageByteOrder_;          //!< Byte order for the image
-
-    }; // class TiffIfdMakernote
-
     //! Header of an Olympus Makernote
     class OlympusMnHeader : public MnHeader {
     public:
@@ -368,7 +176,7 @@ namespace Exiv2 {
         //! Default constructor
         OlympusMnHeader();
         //! Virtual destructor.
-        virtual ~OlympusMnHeader() {}
+        virtual ~OlympusMnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -378,9 +186,9 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return header_.size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return size_; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
         //@}
 
     private:
@@ -398,7 +206,7 @@ namespace Exiv2 {
         //! Default constructor
         Olympus2MnHeader();
         //! Virtual destructor.
-        virtual ~Olympus2MnHeader() {}
+        virtual ~Olympus2MnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -408,10 +216,10 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return header_.size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return size_; }
-        virtual uint32_t baseOffset(uint32_t mnOffset) const { return mnOffset; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
+        virtual uint32_t baseOffset(uint32_t mnOffset) const;
         //@}
 
     private:
@@ -429,7 +237,7 @@ namespace Exiv2 {
         //! Default constructor
         FujiMnHeader();
         //! Virtual destructor.
-        virtual ~FujiMnHeader() {}
+        virtual ~FujiMnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -440,11 +248,11 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t  size()      const { return header_.size_; }
-        virtual uint32_t  write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t  ifdOffset() const { return start_; }
-        virtual ByteOrder byteOrder() const { return byteOrder_; }
-        virtual uint32_t  baseOffset(uint32_t mnOffset) const { return mnOffset; }
+        virtual uint32_t  size() const;
+        virtual uint32_t  write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t  ifdOffset() const;
+        virtual ByteOrder byteOrder() const;
+        virtual uint32_t  baseOffset(uint32_t mnOffset) const;
         //@}
 
     private:
@@ -464,7 +272,7 @@ namespace Exiv2 {
         //! Default constructor
         Nikon2MnHeader();
         //! Virtual destructor.
-        virtual ~Nikon2MnHeader() {}
+        virtual ~Nikon2MnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -474,9 +282,9 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return start_; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
         //@}
 
     private:
@@ -495,7 +303,7 @@ namespace Exiv2 {
         //! Default constructor
         Nikon3MnHeader();
         //! Virtual destructor.
-        virtual ~Nikon3MnHeader() {}
+        virtual ~Nikon3MnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -506,11 +314,11 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t  size()      const { return size_; }
-        virtual uint32_t  write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t  ifdOffset() const { return start_; }
-        virtual ByteOrder byteOrder() const { return byteOrder_; }
-        virtual uint32_t  baseOffset(uint32_t mnOffset) const { return mnOffset + 10; }
+        virtual uint32_t  size()      const;
+        virtual uint32_t  write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t  ifdOffset() const;
+        virtual ByteOrder byteOrder() const;
+        virtual uint32_t  baseOffset(uint32_t mnOffset) const;
         //@}
 
     private:
@@ -530,7 +338,7 @@ namespace Exiv2 {
         //! Default constructor
         PanasonicMnHeader();
         //! Virtual destructor.
-        virtual ~PanasonicMnHeader() {}
+        virtual ~PanasonicMnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -540,9 +348,9 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return start_; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
         //@}
 
     private:
@@ -561,7 +369,7 @@ namespace Exiv2 {
         //! Default constructor
         PentaxMnHeader();
         //! Virtual destructor.
-        virtual ~PentaxMnHeader() {}
+        virtual ~PentaxMnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -571,9 +379,9 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return header_.size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return size_; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
         //@}
 
     private:
@@ -583,15 +391,13 @@ namespace Exiv2 {
 
     }; // class PentaxMnHeader
 
-    //! Header of a Sigma Makernote
-    class SigmaMnHeader : public MnHeader {
+    //! Header of a Samsung Makernote, only used for the relative offset
+    class SamsungMnHeader : public MnHeader {
     public:
         //! @name Creators
         //@{
         //! Default constructor
-        SigmaMnHeader();
-        //! Virtual destructor.
-        virtual ~SigmaMnHeader() {}
+        SamsungMnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -601,9 +407,34 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return start_; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t baseOffset(uint32_t mnOffset) const;
+        //@}
+
+    }; // class SamsungMnHeader
+
+    //! Header of a Sigma Makernote
+    class SigmaMnHeader : public MnHeader {
+    public:
+        //! @name Creators
+        //@{
+        //! Default constructor
+        SigmaMnHeader();
+        //! Virtual destructor.
+        virtual ~SigmaMnHeader();
+        //@}
+        //! @name Manipulators
+        //@{
+        virtual bool read(const byte* pData,
+                          uint32_t    size,
+                          ByteOrder   byteOrder);
+        //@}
+        //! @name Accessors
+        //@{
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
         //@}
 
     private:
@@ -623,7 +454,7 @@ namespace Exiv2 {
         //! Default constructor
         SonyMnHeader();
         //! Virtual destructor.
-        virtual ~SonyMnHeader() {}
+        virtual ~SonyMnHeader();
         //@}
         //! @name Manipulators
         //@{
@@ -633,9 +464,9 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        virtual uint32_t size()      const { return size_; }
-        virtual uint32_t write(Blob& blob, ByteOrder byteOrder) const;
-        virtual uint32_t ifdOffset() const { return start_; }
+        virtual uint32_t size() const;
+        virtual uint32_t write(IoWrapper& ioWrapper, ByteOrder byteOrder) const;
+        virtual uint32_t ifdOffset() const;
         //@}
 
     private:
@@ -651,125 +482,178 @@ namespace Exiv2 {
 
     //! Function to create a simple IFD makernote (Canon, Minolta, Nikon1)
     TiffComponent* newIfdMn(uint16_t    tag,
-                            uint16_t    group,
-                            uint16_t    mnGroup,
+                            IfdId       group,
+                            IfdId       mnGroup,
                             const byte* pData,
                             uint32_t    size,
                             ByteOrder   byteOrder);
 
     //! Function to create a simple IFD makernote (Canon, Minolta, Nikon1)
     TiffComponent* newIfdMn2(uint16_t tag,
-                             uint16_t group,
-                             uint16_t mnGroup);
+                             IfdId    group,
+                             IfdId    mnGroup);
 
     //! Function to create an Olympus makernote
     TiffComponent* newOlympusMn(uint16_t    tag,
-                                uint16_t    group,
-                                uint16_t    mnGroup,
+                                IfdId       group,
+                                IfdId       mnGroup,
                                 const byte* pData,
                                 uint32_t    size,
                                 ByteOrder   byteOrder);
 
     //! Function to create an Olympus makernote
     TiffComponent* newOlympusMn2(uint16_t tag,
-                                 uint16_t group,
-                                 uint16_t mnGroup);
+                                 IfdId    group,
+                                 IfdId    mnGroup);
 
     //! Function to create an Olympus II makernote
     TiffComponent* newOlympus2Mn2(uint16_t tag,
-                                 uint16_t group,
-                                 uint16_t mnGroup);
+                                 IfdId     group,
+                                 IfdId     mnGroup);
 
     //! Function to create a Fujifilm makernote
     TiffComponent* newFujiMn(uint16_t    tag,
-                             uint16_t    group,
-                             uint16_t    mnGroup,
+                             IfdId       group,
+                             IfdId       mnGroup,
                              const byte* pData,
                              uint32_t    size,
                              ByteOrder   byteOrder);
 
     //! Function to create a Fujifilm makernote
     TiffComponent* newFujiMn2(uint16_t tag,
-                              uint16_t group,
-                              uint16_t mnGroup);
+                              IfdId    group,
+                              IfdId    mnGroup);
 
     /*!
       @brief Function to create a Nikon makernote. This will create the
              appropriate Nikon 1, 2 or 3 makernote, based on the arguments.
      */
     TiffComponent* newNikonMn(uint16_t    tag,
-                              uint16_t    group,
-                              uint16_t    mnGroup,
+                              IfdId       group,
+                              IfdId       mnGroup,
                               const byte* pData,
                               uint32_t    size,
                               ByteOrder   byteOrder);
 
     //! Function to create a Nikon2 makernote
     TiffComponent* newNikon2Mn2(uint16_t tag,
-                                uint16_t group,
-                                uint16_t mnGroup);
+                                IfdId    group,
+                                IfdId    mnGroup);
 
     //! Function to create a Nikon3 makernote
     TiffComponent* newNikon3Mn2(uint16_t tag,
-                                uint16_t group,
-                                uint16_t mnGroup);
+                                IfdId    group,
+                                IfdId    mnGroup);
 
     //! Function to create a Panasonic makernote
     TiffComponent* newPanasonicMn(uint16_t    tag,
-                                  uint16_t    group,
-                                  uint16_t    mnGroup,
+                                  IfdId       group,
+                                  IfdId       mnGroup,
                                   const byte* pData,
                                   uint32_t    size,
                                   ByteOrder   byteOrder);
 
     //! Function to create a Panasonic makernote
     TiffComponent* newPanasonicMn2(uint16_t tag,
-                                   uint16_t group,
-                                   uint16_t mnGroup);
+                                   IfdId    group,
+                                   IfdId    mnGroup);
 
     //! Function to create an Pentax makernote
     TiffComponent* newPentaxMn(uint16_t    tag,
-                               uint16_t    group,
-                               uint16_t    mnGroup,
+                               IfdId       group,
+                               IfdId       mnGroup,
                                const byte* pData,
                                uint32_t    size,
                                ByteOrder   byteOrder);
 
     //! Function to create an Pentax makernote
     TiffComponent* newPentaxMn2(uint16_t tag,
-                                uint16_t group,
-                                uint16_t mnGroup);
+                                IfdId    group,
+                                IfdId    mnGroup);
+
+    //! Function to create a Samsung makernote
+    TiffComponent* newSamsungMn(uint16_t    tag,
+                                IfdId       group,
+                                IfdId       mnGroup,
+                                const byte* pData,
+                                uint32_t    size,
+                                ByteOrder   byteOrder);
+
+    //! Function to create a Samsung makernote
+    TiffComponent* newSamsungMn2(uint16_t tag,
+                                 IfdId    group,
+                                 IfdId    mnGroup);
 
     //! Function to create a Sigma makernote
     TiffComponent* newSigmaMn(uint16_t    tag,
-                              uint16_t    group,
-                              uint16_t    mnGroup,
+                              IfdId       group,
+                              IfdId       mnGroup,
                               const byte* pData,
                               uint32_t    size,
                               ByteOrder   byteOrder);
 
     //! Function to create a Sigma makernote
     TiffComponent* newSigmaMn2(uint16_t tag,
-                               uint16_t group,
-                               uint16_t mnGroup);
+                               IfdId    group,
+                               IfdId    mnGroup);
 
     //! Function to create a Sony makernote
     TiffComponent* newSonyMn(uint16_t    tag,
-                             uint16_t    group,
-                             uint16_t    mnGroup,
+                             IfdId       group,
+                             IfdId       mnGroup,
                              const byte* pData,
                              uint32_t    size,
                              ByteOrder   byteOrder);
 
     //! Function to create a Sony1 makernote
     TiffComponent* newSony1Mn2(uint16_t tag,
-                               uint16_t group,
-                               uint16_t mnGroup);
+                               IfdId    group,
+                               IfdId    mnGroup);
 
     //! Function to create a Sony2 makernote
     TiffComponent* newSony2Mn2(uint16_t tag,
-                               uint16_t group,
-                               uint16_t mnGroup);
+                               IfdId    group,
+                               IfdId    mnGroup);
+
+    /*!
+      @brief Function to select cfg + def of the Sony Camera Settings complex binary array.
+
+      @param tag Tag number of the binary array
+      @param pData Pointer to the raw array data.
+      @param size Size of the array data.
+      @param pRoot Pointer to the root component of the TIFF tree.
+      @return An index into the array set, -1 if no match was found.
+     */
+    int sonyCsSelector(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
+
+    /*!
+      @brief Function to select cfg + def of a Nikon complex binary array.
+
+      @param tag Tag number of the binary array
+      @param pData Pointer to the raw array data.
+      @param size Size of the array data.
+      @param pRoot Pointer to the root component of the TIFF tree.
+      @return An index into the array set, -1 if no match was found.
+     */
+    int nikonSelector(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
+
+    /*!
+      @brief Encrypt and decrypt Nikon data.
+
+      Checks the version of the Nikon data array and en/decrypts (portions of) it as
+      needed. (The Nikon encryption algorithm is symmetric.)
+
+      @note This function requires access to other components of the composite, it
+            should only be called after all other components are read.
+
+      @param tag Tag number of the binary array
+      @param pData Pointer to the start of the data to en/decrypt.
+      @param size Size of the data buffer.
+      @param pRoot Pointer to the root element of the composite.
+      @return En/decrypted data. Ownership of the memory is passed to the caller.
+              The buffer may be empty in case no decryption was needed.
+     */
+    DataBuf nikonCrypt(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
 
 }}                                      // namespace Internal, Exiv2
 
