@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,22 +20,22 @@
  */
 /*
   File:      pentaxmn.cpp
-  Version:   $Rev: 1756 $
+  Version:   $Rev: 2455 $
   Author(s): Michal Cihar <michal@cihar.com>
   Based on fujimn.cpp by:
              Andreas Huggel (ahu) <ahuggel@gmx.net>
-             Gilles Caulier (gc) <caulier.gilles@kdemail.net>
+             Gilles Caulier (gc) <caulier dot gilles at gmail dot com>
   History:   27-Sep-07 created
   Credits:   See header file.
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: pentaxmn.cpp 1756 2009-03-05 10:20:26Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: pentaxmn.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
 #include "types.hpp"
-#include "pentaxmn.hpp"
+#include "pentaxmn_int.hpp"
 #include "value.hpp"
 #include "i18n.h"                // NLS support.
 
@@ -49,6 +49,7 @@ EXIV2_RCSID("@(#) $Id: pentaxmn.cpp 1756 2009-03-05 10:20:26Z ahuggel $")
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
+    namespace Internal {
 
     //! ShootingMode, tag 0x0001
     extern const TagDetails pentaxShootingMode[] = {
@@ -100,33 +101,50 @@ namespace Exiv2 {
         {    0x12b9c, N_("K100D") },
         {    0x12b9d, N_("K110D") },
         {    0x12ba2, N_("K100D Super") },
-        {    0x12bb0, N_("Optio T10") },
+        {    0x12bb0, N_("Optio T10/T20") },
         {    0x12be2, N_("Optio W10") },
         {    0x12bf6, N_("Optio M10") },
         {    0x12c1e, N_("K10D") },
         {    0x12c20, N_("Samsung GX10") },
         {    0x12c28, N_("Optio S7") },
+        {    0x12c2d, N_("Optio L20") },
         {    0x12c32, N_("Optio M20") },
         {    0x12c3c, N_("Optio W20") },
         {    0x12c46, N_("Optio A20") },
         {    0x12c8c, N_("Optio M30") },
         {    0x12c78, N_("Optio E30") },
+        {    0x12c7d, N_("Optio E35") },
         {    0x12c82, N_("Optio T30") },
         {    0x12c96, N_("Optio W30") },
         {    0x12ca0, N_("Optio A30") },
         {    0x12cb4, N_("Optio E40") },
         {    0x12cbe, N_("Optio M40") },
         {    0x12cc8, N_("Optio Z10") },
-        {    0x12cdc, N_("Optio S10") },
-        {    0x12ce6, N_("Optio A40") },
-        {    0x12cf0, N_("Optio V10") },
         {    0x12cd2, N_("K20D") },
+        {    0x12cd4, N_("Samsung GX20") },
         {    0x12cdc, N_("Optio S10") },
         {    0x12ce6, N_("Optio A40") },
         {    0x12cf0, N_("Optio V10") },
         {    0x12cfa, N_("K200D") },
+        {    0x12d04, N_("Optio S12") },
         {    0x12d0e, N_("Optio E50") },
         {    0x12d18, N_("Optio M50") },
+        {    0x12d2c, N_("Optio V20") },
+        {    0x12d40, N_("Optio W60") },
+        {    0x12d4a, N_("Optio M60") },
+        {    0x12d68, N_("Optio E60") },
+        {    0x12d72, N_("K2000") },
+        {    0x12d73, N_("K-m") },
+        {    0x12d86, N_("Optio P70") },
+        {    0x12d9a, N_("Optio E70") },
+        {    0x12dae, N_("X70") },
+        {    0x12db8, N_("K-7") },
+        {    0x12dcc, N_("Optio W80") },
+        {    0x12dea, N_("Optio P80") },
+        {    0x12df4, N_("Optio WS80") },
+        {    0x12dfe, N_("K-x") },
+        {    0x12e08, N_("645D") },
+        {    0x12e3a, N_("Optio I-10") },
     };
 
     //! Quality, tag 0x0008
@@ -181,15 +199,19 @@ namespace Exiv2 {
         {    1, N_("Macro") },
         {    2, N_("Infinity") },
         {    3, N_("Manual") },
+        {    4, N_("Super Macro") },
         {    5, N_("Pan Focus") },
         {    16, N_("AF-S") },
         {    17, N_("AF-C") },
+        {    18, N_("AF-A") },
     };
 
     //! AFPoint, tag 0x000e
     extern const TagDetails pentaxAFPoint[] = {
         {    0xffff, N_("Auto") },
         {    0xfffe, N_("Fixed Center") },
+        {    0xfffd, N_("Automatic Tracking AF") },
+        {    0xfffc, N_("Face Recognition AF") },
         {    1, N_("Upper-left") },
         {    2, N_("Top") },
         {    3, N_("Upper-right") },
@@ -203,6 +225,21 @@ namespace Exiv2 {
         {    11, N_("Lower-right") },
     };
 
+    //! AFPointInFocus, tag 0x000f
+    extern const TagDetails pentaxAFPointFocus[] = {
+        {    0xffff, N_("None") },
+        {    0, N_("Fixed Center or multiple") },
+        {    1, N_("Top-left") },
+        {    2, N_("Top-center") },
+        {    3, N_("Top-right") },
+        {    4, N_("Left") },
+        {    5, N_("Center") },
+        {    6, N_("Right") },
+        {    7, N_("Bottom-left") },
+        {    8, N_("Bottom-center") },
+        {    9, N_("Bottom-right") },
+    };
+
     //! ISO, tag 0x0014
     extern const TagDetails pentaxISO[] = {
         {    3, N_("50") },
@@ -221,7 +258,12 @@ namespace Exiv2 {
         {    16, N_("1000") },
         {    17, N_("1250") },
         {    18, N_("1600") },
+        {    19, N_("2000") },
+        {    20, N_("2500") },
         {    21, N_("3200") },
+        {    22, N_("4000") },
+        {    23, N_("5000") },
+        {    24, N_("6400") },
         {    50, N_("50") },
         {    100, N_("100") },
         {    200, N_("200") },
@@ -230,6 +272,19 @@ namespace Exiv2 {
         {    800, N_("800") },
         {    1600, N_("1600") },
         {    3200, N_("3200") },
+        {    258, N_("50") },
+        {    259, N_("70") },
+        {    260, N_("100") },
+        {    261, N_("140") },
+        {    262, N_("200") },
+        {    263, N_("280") },
+        {    264, N_("400") },
+        {    265, N_("560") },
+        {    266, N_("800") },
+        {    267, N_("1100") },
+        {    268, N_("1600") },
+        {    269, N_("2200") },
+        {    270, N_("3200") },
     };
 
     //! Generic for Off/On switches
@@ -264,6 +319,7 @@ namespace Exiv2 {
         {    8, N_("WhiteFluorescent") },
         {    9, N_("Flash") },
         {    10, N_("Cloudy") },
+        {    17, N_("Kelvin") },
         {    65534, N_("Unknown") },
         {    65535, N_("User Selected") },
     };
@@ -283,13 +339,15 @@ namespace Exiv2 {
 
     //! Saturation, tag 0x001f
     extern const TagDetails pentaxSaturation[] = {
-        {    0, N_("Low") },
-        {    1, N_("Normal") },
-        {    2, N_("High") },
-        {    3, N_("Med Low") },
-        {    4, N_("Med High") },
-        {    5, N_("Very Low") },
-        {    6, N_("Very High") },
+        {     0, N_("Low")       },
+        {     1, N_("Normal")    },
+        {     2, N_("High")      },
+        {     3, N_("Med Low")   },
+        {     4, N_("Med High")  },
+        {     5, N_("Very Low")  },
+        {     6, N_("Very High") },
+        { 65535, N_("None")      },
+        { 65535, N_("None")      }              // To silence compiler warning
     };
 
     //! Contrast, tag 0x0020
@@ -393,6 +451,7 @@ namespace Exiv2 {
         {    68, N_("Amsterdam") },
         {    69, N_("Stockholm") },
         {    70, N_("Lisbon") },
+        {    71, N_("Copenhagen") },
     };
 
     //! ImageProcessing, combi-tag 0x0032 (4 bytes)
@@ -424,6 +483,9 @@ namespace Exiv2 {
         { 0x001000, N_("Pet") },
         { 0x001100, N_("Candlelight") },
         { 0x001200, N_("Museum") },
+        { 0x001300, N_("Food") },
+        { 0x001400, N_("Stage Lighting") },
+        { 0x001500, N_("Night Snap") },
         /* AUTO PICT modes (auto-selected) */
         { 0x010400, N_("Auto PICT (Standard)") },
         { 0x010500, N_("Auto PICT (Portrait)") },
@@ -461,7 +523,9 @@ namespace Exiv2 {
         { 0x100000, N_("Flash X-Sync Speed AE") },
         { 0x100001, N_("Flash X-Sync Speed AE (1)") },
         /* other modes */
-        { 0x000001, N_("Program") }
+        { 0x000001, N_("Program") },
+        { 0xfe0000, N_("Video (30 fps)") },
+        { 0xff0004, N_("Video (24 fps)") },
     };
 
     //! DriveMode, combi-tag 0x0034 (4 bytes)
@@ -470,11 +534,13 @@ namespace Exiv2 {
         { 0x01000000, N_("Continuous") },
         { 0x02000000, N_("Continuous (Hi)") },
         { 0x03000000, N_("Burst") },
+        { 0x00100000, N_("Single-frame") }, /* on 645D */
         { 0x00010000, N_("Self-timer (12 sec)") },
         { 0x00020000, N_("Self-timer (2 sec)") },
         { 0x00000100, N_("Remote Control (3 sec)") },
         { 0x00000200, N_("Remote Control") },
         { 0x00000001, N_("Multiple Exposure") },
+        { 0x000000ff, N_("Video") },
     };
 
     //! ColorSpace, tag 0x0037
@@ -599,21 +665,36 @@ namespace Exiv2 {
         { 0x060d, "smc PENTAX-FA* 400mm F5.6 ED[IF]" },
         { 0x060e, "smc PENTAX-FA* MACRO 200mm F4 ED[IF]" },
         { 0x0700, "smc PENTAX-DA 21mm F3.2 AL Limited" },
+        { 0x074b, "Tamron SP AF 70-200mm F2.8 Di LD [IF] Macro (A001)" },
+        { 0x07d9, "smc PENTAX-DA 50-200mm F4-5.6 ED WR" },
+        { 0x07da, "smc PENTAX-DA 18-55mm F3.5-5.6 AL WR" },
+        { 0x07dc, "Tamron SP AF 10-24mm F3.5-4.5 Di II LD Aspherical [IF]" },
+        { 0x07de, "smc PENTAX-DA 18-55mm F3.5-5.6 AL II" },
+        { 0x07df, "Samsung D-XENON 18-55mm F3.5-5.6 II" },
+        { 0x07e0, "smc PENTAX-DA 15mm F4 ED AL Limited" },
+        { 0x07e1, "Samsung D-XENON 18-250mm F3.5-6.3" },
         { 0x07e5, "smc PENTAX-DA 18-55mm F3.5-5.6 AL II" },
         { 0x07e6, "Tamron AF 17-50mm F2.8 XR Di-II LD (Model A16)" },
         { 0x07e7, "smc PENTAX-DA 18-250mm F3.5-6.3ED AL [IF]" },
         { 0x07e9, "smc PENTAX-DA 35mm F2.8 Macro Limited" },
         { 0x07ea, "smc PENTAX-DA* 300 mm F4ED [IF] SDM (SDM not used)" },
         { 0x07eb, "smc PENTAX-DA* 200mm F2.8 ED [IF] SDM (SDM not used)" },
+        { 0x07ec, "smc PENTAX-DA 55-300mm F4-5.8 ED" },
         { 0x07ee, "TAMRON AF 18-250mm F3.5-6.3 Di II LD Aspherical [IF] MACRO" },
         { 0x07f1, "smc PENTAX-DA* 50-135mm F2.8 ED [IF] SDM (SDM not used)" },
         { 0x07f2, "smc PENTAX-DA* 16-50mm F2.8 ED AL [IF] SDM (SDM not used)" },
         { 0x07f3, "smc PENTAX-DA 70mm F2.4 Limited" },
         { 0x07f4, "smc PENTAX-DA 21mm F3.2 AL Limited" },
-        { 0x08ea, "smc PENTAX-DA* 300 mm F4ED [IF] SDM" },
+        { 0x08e2, "smc PENTAX-DA* 55mm F1.4 SDM" },
+        { 0x08e3, "smc PENTAX DA* 60-250mm F4 [IF] SDM"},
+        { 0x08e8, "smc PENTAX-DA 17-70mm F4 AL [IF] SDM" },
+        { 0x08ea, "smc PENTAX-DA* 300mm F4 ED [IF] SDM" },
         { 0x08eb, "smc PENTAX-DA* 200mm F2.8 ED [IF] SDM" },
         { 0x08f1, "smc PENTAX-DA* 50-135mm F2.8 ED [IF] SDM" },
         { 0x08f2, "smc PENTAX-DA* 16-50mm F2.8 ED AL [IF] SDM" },
+        { 0x0b04, "smc PENTAX-FA645 Zoom 45mm-85mm F4.5" },
+        { 0x0b07, "smc PENTAX-FA645 Macro 120mm F4" },
+        { 0x0b11, "smc PENTAX-FA645 Zoom 150mm-300mm F5.6 ED [IF]" },
     };
 
     //! ImageTone, tag 0x004f
@@ -624,6 +705,7 @@ namespace Exiv2 {
         {    3, N_("Landscape") },
         {    4, N_("Vibrant") },
         {    5, N_("Monochrome") },
+        {    7, N_("Reversal film") },
     };
 
     //! DynamicRangeExpansion, tag 0x0069
@@ -638,6 +720,7 @@ namespace Exiv2 {
         {   1, N_("Weakest") },
         {   2, N_("Weak") },
         {   3, N_("Strong") },
+        {   4, N_("Custom") },
     };
 
     std::ostream& PentaxMakerNote::printPentaxVersion(std::ostream& os, const Value& value, const ExifData*)
@@ -781,244 +864,266 @@ namespace Exiv2 {
     const TagInfo PentaxMakerNote::tagInfo_[] = {
         TagInfo(0x0000, "Version", N_("Version"),
                 N_("Pentax Makernote version"),
-                pentaxIfdId, makerTags, undefined, printPentaxVersion),
+                pentaxId, makerTags, undefined, -1, printPentaxVersion),
         TagInfo(0x0001, "Mode", N_("Shooting mode"),
                 N_("Camera shooting mode"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxShootingMode)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxShootingMode)),
         TagInfo(0x0002, "PreviewResolution", N_("Resolution of a preview image"),
                 N_("Resolution of a preview image"),
-                pentaxIfdId, makerTags, undefined, printPentaxResolution),
+                pentaxId, makerTags, undefined, -1, printPentaxResolution),
         TagInfo(0x0003, "PreviewLength", N_("Length of a preview image"),
                 N_("Size of an IFD containing a preview image"),
-                pentaxIfdId, makerTags, undefined, printValue),
+                pentaxId, makerTags, undefined, -1, printValue),
         TagInfo(0x0004, "PreviewOffset", N_("Pointer to a preview image"),
                 N_("Offset to an IFD containing a preview image"),
-                pentaxIfdId, makerTags, undefined, printValue),
+                pentaxId, makerTags, undefined, -1, printValue),
         TagInfo(0x0005, "ModelID", N_("Model identification"),
                 N_("Pentax model idenfication"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxModel)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxModel)),
         TagInfo(0x0006, "Date", N_("Date"),
                 N_("Date"),
-                pentaxIfdId, makerTags, undefined, printPentaxDate),
+                pentaxId, makerTags, undefined, -1, printPentaxDate),
         TagInfo(0x0007, "Time", N_("Time"),
                 N_("Time"),
-                pentaxIfdId, makerTags, undefined, printPentaxTime),
+                pentaxId, makerTags, undefined, -1, printPentaxTime),
         TagInfo(0x0008, "Quality", N_("Image quality"),
                 N_("Image quality settings"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxQuality)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxQuality)),
         TagInfo(0x0009, "Size", N_("Image size"),
                 N_("Image size settings"),
-                pentaxIfdId, makerTags, unsignedLong, EXV_PRINT_TAG(pentaxSize)),
+                pentaxId, makerTags, unsignedLong, -1, EXV_PRINT_TAG(pentaxSize)),
         /* Some missing ! */
         TagInfo(0x000c, "Flash", N_("Flash mode"),
                 N_("Flash mode settings"),
-                pentaxIfdId, makerTags, unsignedLong, EXV_PRINT_TAG(pentaxFlash)),
+                pentaxId, makerTags, unsignedLong, -1, EXV_PRINT_TAG(pentaxFlash)),
         TagInfo(0x000d, "Focus", N_("Focus mode"),
                 N_("Focus mode settings"),
-                pentaxIfdId, makerTags, unsignedLong, EXV_PRINT_TAG(pentaxFocus)),
+                pentaxId, makerTags, unsignedLong, -1, EXV_PRINT_TAG(pentaxFocus)),
         TagInfo(0x000e, "AFPoint", N_("AF point"),
                 N_("Selected AF point"),
-                pentaxIfdId, makerTags, unsignedLong, EXV_PRINT_TAG(pentaxAFPoint)),
+                pentaxId, makerTags, unsignedLong, -1, EXV_PRINT_TAG(pentaxAFPoint)),
+        TagInfo(0x000F, "AFPointInFocus", N_("AF point in focus"),
+                N_("AF point in focus"),
+                pentaxId, makerTags, unsignedLong, -1, EXV_PRINT_TAG(pentaxAFPointFocus)),
         /* Some missing ! */
         TagInfo(0x0012, "ExposureTime", N_("Exposure time"),
                 N_("Exposure time"),
-                pentaxIfdId, makerTags, unsignedLong, printPentaxExposure),
+                pentaxId, makerTags, unsignedLong, -1, printPentaxExposure),
         TagInfo(0x0013, "FNumber", N_("F-Number"),
                 N_("F-Number"),
-                pentaxIfdId, makerTags, unsignedLong, printPentaxFValue),
+                pentaxId, makerTags, unsignedLong, -1, printPentaxFValue),
         TagInfo(0x0014, "ISO", N_("ISO sensitivity"),
                 N_("ISO sensitivity settings"),
-                pentaxIfdId, makerTags, unsignedLong, EXV_PRINT_TAG(pentaxISO)),
+                pentaxId, makerTags, unsignedLong, -1, EXV_PRINT_TAG(pentaxISO)),
         /* Some missing ! */
         TagInfo(0x0016, "ExposureCompensation", N_("Exposure compensation"),
                 N_("Exposure compensation"),
-                pentaxIfdId, makerTags, unsignedLong, printPentaxCompensation),
+                pentaxId, makerTags, unsignedLong, -1, printPentaxCompensation),
         /* Some missing ! */
         TagInfo(0x0017, "MeteringMode", N_("MeteringMode"),
                 N_("MeteringMode"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxMeteringMode)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxMeteringMode)),
         TagInfo(0x0018, "AutoBracketing", N_("AutoBracketing"),
                 N_("AutoBracketing"),
-                pentaxIfdId, makerTags, undefined, printPentaxBracketing),
+                pentaxId, makerTags, undefined, -1, printPentaxBracketing),
         TagInfo(0x0019, "WhiteBallance", N_("White ballance"),
                 N_("White ballance"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxWhiteBallance)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxWhiteBallance)),
         TagInfo(0x001a, "WhiteBallanceMode", N_("White ballance mode"),
                 N_("White ballance mode"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxWhiteBallanceMode)),
-        /* Some missing ! */
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxWhiteBallanceMode)),
+        TagInfo(0x001b, "BlueBalance", N_("Blue balance"),
+                N_("Blue color balance"),
+                pentaxId, makerTags, unsignedLong, -1, printValue),
+        TagInfo(0x001c, "RedBalance", N_("Red balance"),
+                N_("Red color balance"),
+                pentaxId, makerTags, unsignedLong, -1, printValue),
         TagInfo(0x001d, "FocalLength", N_("FocalLength"),
                 N_("FocalLength"),
-                pentaxIfdId, makerTags, undefined, printPentaxFocalLength),
-        /* Some missing ! */
+                pentaxId, makerTags, undefined, -1, printPentaxFocalLength),
+        TagInfo(0x001e, "DigitalZoom", N_("Digital zoom"),
+                N_("Digital zoom"),
+                pentaxId, makerTags, unsignedLong, -1, printValue),
         TagInfo(0x001f, "Saturation", N_("Saturation"),
                 N_("Saturation"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxSaturation)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxSaturation)),
         TagInfo(0x0020, "Contrast", N_("Contrast"),
                 N_("Contrast"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxContrast)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxContrast)),
         TagInfo(0x0021, "Sharpness", N_("Sharpness"),
                 N_("Sharpness"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxSharpness)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxSharpness)),
         TagInfo(0x0022, "Location", N_("Location"),
                 N_("Location"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxLocation)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxLocation)),
         TagInfo(0x0023, "Hometown", N_("Hometown"),
                 N_("Home town"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxCities)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxCities)),
         TagInfo(0x0024, "Destination", N_("Destination"),
                 N_("Destination"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxCities)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxCities)),
         TagInfo(0x0025, "HometownDST", N_("Hometown DST"),
                 N_("Whether day saving time is active in home town"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxYesNo)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxYesNo)),
         TagInfo(0x0026, "DestinationDST", N_("Destination DST"),
                 N_("Whether day saving time is active in destination"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_TAG(pentaxYesNo)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_TAG(pentaxYesNo)),
         TagInfo(0x0027, "DSPFirmwareVersion", N_("DSPFirmwareVersion"),
                 N_("DSPFirmwareVersion"),
-                pentaxIfdId, makerTags, unsignedByte, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, unsignedByte, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0028, "CPUFirmwareVersion", N_("CPUFirmwareVersion"),
                 N_("CPUFirmwareVersion"),
-                pentaxIfdId, makerTags, unsignedByte, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, unsignedByte, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0029, "FrameNumber", N_("Frame number"),
                 N_("Frame number"),
-                pentaxIfdId, makerTags, undefined, printValue),
+                pentaxId, makerTags, undefined, -1, printValue),
         /* Some missing ! */
         TagInfo(0x002d, "EffectiveLV", N_("Light value"),
                 N_("Camera calculated light value, includes exposure compensation"),
-                pentaxIfdId, makerTags, unsignedShort, printValue),
+                pentaxId, makerTags, unsignedShort, -1, printValue),
         /* Some missing ! */
         TagInfo(0x0032, "ImageProcessing", N_("Image processing"),
                 N_("Image processing"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_COMBITAG(pentaxImageProcessing, 4, 0)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_COMBITAG(pentaxImageProcessing, 4, 0)),
         TagInfo(0x0033, "PictureMode", N_("Picture mode"),
                 N_("Picture mode"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_COMBITAG(pentaxPictureMode, 3, 0)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_COMBITAG(pentaxPictureMode, 3, 0)),
         TagInfo(0x0034, "DriveMode", N_("Drive mode"),
                 N_("Drive mode"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_COMBITAG(pentaxDriveMode, 4, 0)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_COMBITAG(pentaxDriveMode, 4, 0)),
         /* Some missing ! */
         TagInfo(0x0037, "ColorSpace", N_("Color space"),
                 N_("Color space"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxColorSpace)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxColorSpace)),
+        TagInfo(0x0038, "ImageAreaOffset", N_("Image area offset"),
+                N_("Image area offset"),
+                pentaxId, makerTags, unsignedLong, -1, printValue),
+        TagInfo(0x0039, "RawImageSize", N_("Raw image size"),
+                N_("Raw image size"),
+                pentaxId, makerTags, unsignedLong, -1, printValue),
         /* Some missing ! */
         TagInfo(0x003e, "PreviewImageBorders", N_("Preview image borders"),
                 N_("Preview image borders"),
-                pentaxIfdId, makerTags, unsignedByte, printValue),
+                pentaxId, makerTags, unsignedByte, -1, printValue),
         TagInfo(0x003f, "LensType", N_("Lens type"),
                 N_("Lens type"),
-                pentaxIfdId, makerTags, unsignedByte, EXV_PRINT_COMBITAG(pentaxLensType, 2, 1)),
+                pentaxId, makerTags, unsignedByte, -1, EXV_PRINT_COMBITAG_MULTI(pentaxLensType, 2, 1, 2)),
         TagInfo(0x0040, "SensitivityAdjust", N_("Sensitivity adjust"),
                 N_("Sensitivity adjust"),
-                pentaxIfdId, makerTags, unsignedLong, printValue),
+                pentaxId, makerTags, unsignedLong, -1, printValue),
         TagInfo(0x0041, "DigitalFilter", N_("Digital filter"),
                 N_("Digital filter"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxOffOn)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxOffOn)),
         /* Some missing ! */
         TagInfo(0x0047, "Temperature", N_("Temperature"),
                 N_("Camera temperature"),
-                pentaxIfdId, makerTags, signedByte, printPentaxTemperature),
+                pentaxId, makerTags, signedByte, -1, printPentaxTemperature),
         TagInfo(0x0048, "AELock", N_("AE lock"),
                 N_("AE lock"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxOffOn)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxOffOn)),
         TagInfo(0x0049, "NoiseReduction", N_("Noise reduction"),
                 N_("Noise reduction"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxOffOn)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxOffOn)),
         /* Some missing ! */
         TagInfo(0x004d, "FlashExposureCompensation", N_("Flash exposure compensation"),
                 N_("Flash exposure compensation"),
-                pentaxIfdId, makerTags, signedLong, printPentaxFlashCompensation),
+                pentaxId, makerTags, signedLong, -1, printPentaxFlashCompensation),
         /* Some missing ! */
         TagInfo(0x004f, "ImageTone", N_("Image tone"),
                 N_("Image tone"),
-                pentaxIfdId, makerTags, unsignedShort, EXV_PRINT_TAG(pentaxImageTone)),
+                pentaxId, makerTags, unsignedShort, -1, EXV_PRINT_TAG(pentaxImageTone)),
         TagInfo(0x0050, "ColorTemperature", N_("Colort temperature"),
                 N_("Colort temperature"),
-                pentaxIfdId, makerTags, unsignedShort, printValue),
+                pentaxId, makerTags, unsignedShort, -1, printValue),
         /* Some missing ! */
         TagInfo(0x005c, "ShakeReduction", N_("Shake reduction"),
                 N_("Shake reduction information"),
-                pentaxIfdId, makerTags, undefined, printValue),
+                pentaxId, makerTags, undefined, -1, printValue),
         TagInfo(0x005d, "ShutterCount", N_("Shutter count"),
                 N_("Shutter count"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: This has some encryption by date (see exiftool) */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: This has some encryption by date (see exiftool) */
         TagInfo(0x0069, "DynamicRangeExpansion", N_("Dynamic range expansion"),
                 N_("Dynamic range expansion"),
-                pentaxIfdId, makerTags, undefined, EXV_PRINT_COMBITAG(pentaxDynamicRangeExpansion, 4, 0)),
+                pentaxId, makerTags, undefined, -1, EXV_PRINT_COMBITAG(pentaxDynamicRangeExpansion, 4, 0)),
         TagInfo(0x0071, "HighISONoiseReduction", N_("High ISO noise reduction"),
                 N_("High ISO noise reduction"),
-                pentaxIfdId, makerTags, unsignedByte, EXV_PRINT_TAG(pentaxHighISONoiseReduction)),
+                pentaxId, makerTags, unsignedByte, -1, EXV_PRINT_TAG(pentaxHighISONoiseReduction)),
+        TagInfo(0x0072, "AFAdjustment", N_("AF Adjustment"),
+                N_("AF Adjustment"),
+                pentaxId, makerTags, undefined, -1, printValue),
         /* Many missing ! */
         TagInfo(0x0200, "BlackPoint", N_("Black point"),
                 N_("Black point"),
-                pentaxIfdId, makerTags, undefined, printValue),
+                pentaxId, makerTags, undefined, -1, printValue),
         TagInfo(0x0201, "WhitePoint", N_("White point"),
                 N_("White point"),
-                pentaxIfdId, makerTags, undefined, printValue),
+                pentaxId, makerTags, undefined, -1, printValue),
         /* Some missing ! */
         TagInfo(0x0205, "ShotInfo", N_("ShotInfo"),
                 N_("ShotInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0206, "AEInfo", N_("AEInfo"),
                 N_("AEInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0207, "LensInfo", N_("LensInfo"),
                 N_("LensInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0208, "FlashInfo", N_("FlashInfo"),
                 N_("FlashInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0209, "AEMeteringSegments", N_("AEMeteringSegments"),
                 N_("AEMeteringSegments"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x020a, "FlashADump", N_("FlashADump"),
                 N_("FlashADump"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x020b, "FlashBDump", N_("FlashBDump"),
                 N_("FlashBDump"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         /* Some missing ! */
         TagInfo(0x020d, "WB_RGGBLevelsDaylight", N_("WB_RGGBLevelsDaylight"),
                 N_("WB_RGGBLevelsDaylight"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x020e, "WB_RGGBLevelsShade", N_("WB_RGGBLevelsShade"),
                 N_("WB_RGGBLevelsShade"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x020f, "WB_RGGBLevelsCloudy", N_("WB_RGGBLevelsCloudy"),
                 N_("WB_RGGBLevelsCloudy"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0210, "WB_RGGBLevelsTungsten", N_("WB_RGGBLevelsTungsten"),
                 N_("WB_RGGBLevelsTungsten"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0211, "WB_RGGBLevelsFluorescentD", N_("WB_RGGBLevelsFluorescentD"),
                 N_("WB_RGGBLevelsFluorescentD"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0212, "WB_RGGBLevelsFluorescentN", N_("WB_RGGBLevelsFluorescentN"),
                 N_("WB_RGGBLevelsFluorescentN"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0213, "WB_RGGBLevelsFluorescentW", N_("WB_RGGBLevelsFluorescentW"),
                 N_("WB_RGGBLevelsFluorescentW"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0214, "WB_RGGBLevelsFlash", N_("WB_RGGBLevelsFlash"),
                 N_("WB_RGGBLevelsFlash"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0215, "CameraInfo", N_("CameraInfo"),
                 N_("CameraInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0216, "BatteryInfo", N_("BatteryInfo"),
                 N_("BatteryInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x021f, "AFInfo", N_("AFInfo"),
                 N_("AFInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
         TagInfo(0x0222, "ColorInfo", N_("ColorInfo"),
                 N_("ColorInfo"),
-                pentaxIfdId, makerTags, undefined, printValue), /* TODO: Decoding missing */
+                pentaxId, makerTags, undefined, -1, printValue), /* TODO: Decoding missing */
+        TagInfo(0x0229, "SerialNumber", N_("Serial Number"),
+                N_("Serial Number"),
+                pentaxId, makerTags, asciiString, -1, printValue),
         // End of list marker
         TagInfo(0xffff, "(UnknownPentaxMakerNoteTag)", "(UnknownPentaxMakerNoteTag)",
                 N_("Unknown PentaxMakerNote tag"),
-                pentaxIfdId, makerTags, invalidTypeId, printValue)
+                pentaxId, makerTags, asciiString, -1, printValue)
     };
 
     const TagInfo* PentaxMakerNote::tagList()
@@ -1026,4 +1131,4 @@ namespace Exiv2 {
         return tagInfo_;
     }
 
-}                                       // namespace Exiv2
+}}                                       // namespace Internal, Exiv2
