@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:    pngchunk.cpp
-  Version: $Rev: 1820 $
+  Version: $Rev: 2455 $
   Author(s): Gilles Caulier (cgilles) <caulier dot gilles at gmail dot com>
   History: 12-Jun-06, gc: submitted
   Credits: See header file
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: pngchunk.cpp 1820 2009-06-16 13:30:36Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: pngchunk.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -103,7 +103,7 @@ namespace Exiv2 {
         std::cout << "Exiv2::PngChunk::decodeTXTChunk: TXT chunk data: "
                   << std::string((const char*)arr.pData_, 32) << "\n";
 #endif
-        parseChunkContent(pImage, key.pData_, arr);
+        parseChunkContent(pImage, key.pData_, key.size_, arr);
 
     } // PngChunk::decodeTXTChunk
 
@@ -111,7 +111,7 @@ namespace Exiv2 {
     {
         // From a tEXt, zTXt, or iTXt chunk,
         // we get the key, it's a null terminated string at the chunk start
-
+        if (data.size_ <= (stripHeader ? 8 : 0)) throw Error(14);
         const byte *key = data.pData_ + (stripHeader ? 8 : 0);
 
         // Find null string at end of key.
@@ -162,7 +162,6 @@ namespace Exiv2 {
             const byte* text = data.pData_ + keysize + 1;
             long textsize    = data.size_  - keysize - 1;
 
-            arr.alloc(textsize);
             arr = DataBuf(text, textsize);
         }
         else if(type == iTXt_Chunk)
@@ -175,10 +174,10 @@ namespace Exiv2 {
             const byte* compressionMethod = data.pData_ + keysize + 2;
             // language description string after the compression technique spec
             std::string languageText((const char*)(data.pData_ + keysize + 3));
-            unsigned int languageTextSize = languageText.size();
+            unsigned int languageTextSize = static_cast<unsigned int>(languageText.size());
             // translated keyword string after the language description
             std::string translatedKeyText((const char*)(data.pData_ + keysize + 3 + languageTextSize +1));
-            unsigned int translatedKeyTextSize = translatedKeyText.size();
+            unsigned int translatedKeyTextSize = static_cast<unsigned int>(translatedKeyText.size());
 
             if ( compressionFlag[0] == 0x00 )
             {
@@ -228,13 +227,17 @@ namespace Exiv2 {
 
     } // PngChunk::parsePngChunk
 
-    void PngChunk::parseChunkContent(Image* pImage, const byte *key, const DataBuf arr)
+    void PngChunk::parseChunkContent(      Image*  pImage,
+                                     const byte*   key,
+                                           long    keySize,
+                                     const DataBuf arr)
     {
         // We look if an ImageMagick EXIF raw profile exist.
 
-        if ( (memcmp("Raw profile type exif", key, 21) == 0 ||
-              memcmp("Raw profile type APP1", key, 21) == 0) &&
-             pImage->exifData().empty())
+        if (   keySize >= 21 
+            && (   memcmp("Raw profile type exif", key, 21) == 0
+                || memcmp("Raw profile type APP1", key, 21) == 0)
+            && pImage->exifData().empty())
         {
             DataBuf exifData = readRawProfile(arr);
             long length      = exifData.size_;
@@ -273,7 +276,7 @@ namespace Exiv2 {
                 else
                 {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Exiv2::PngChunk::parseChunkContent: Failed to decode Exif metadata.\n";
+                    EXV_WARNING << "Failed to decode Exif metadata.\n";
 #endif
                     pImage->exifData().clear();
                 }
@@ -282,7 +285,8 @@ namespace Exiv2 {
 
         // We look if an ImageMagick IPTC raw profile exist.
 
-        if (   memcmp("Raw profile type iptc", key, 21) == 0
+        if (   keySize >= 21
+            && memcmp("Raw profile type iptc", key, 21) == 0
             && pImage->iptcData().empty()) {
             DataBuf psData = readRawProfile(arr);
             if (psData.size_ > 0) {
@@ -313,7 +317,7 @@ namespace Exiv2 {
                                           &iptcBlob[0],
                                           static_cast<uint32_t>(iptcBlob.size()))) {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to decode IPTC metadata.\n";
+                    EXV_WARNING << "Failed to decode IPTC metadata.\n";
 #endif
                     pImage->clearIptcData();
                 }
@@ -323,7 +327,7 @@ namespace Exiv2 {
                                           psData.pData_,
                                           psData.size_)) {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to decode IPTC metadata.\n";
+                    EXV_WARNING << "Failed to decode IPTC metadata.\n";
 #endif
                     pImage->clearIptcData();
                 }
@@ -332,8 +336,9 @@ namespace Exiv2 {
 
         // We look if an ImageMagick XMP raw profile exist.
 
-        if ( memcmp("Raw profile type xmp", key, 20) == 0 &&
-             pImage->xmpData().empty())
+        if (   keySize >= 20
+            && memcmp("Raw profile type xmp", key, 20) == 0
+            && pImage->xmpData().empty()) 
         {
             DataBuf xmpBuf = readRawProfile(arr);
             long length    = xmpBuf.size_;
@@ -346,15 +351,15 @@ namespace Exiv2 {
                 if (idx != std::string::npos && idx > 0)
                 {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Exiv2::PngChunk::parseChunkContent: Removing " << idx
-                              << " characters from the beginning of the XMP packet\n";
+                    EXV_WARNING << "Removing " << idx
+                                << " characters from the beginning of the XMP packet\n";
 #endif
                     xmpPacket = xmpPacket.substr(idx);
                 }
                 if (XmpParser::decode(pImage->xmpData(), xmpPacket))
                 {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Exiv2::PngChunk::parseChunkContent: Failed to decode XMP metadata.\n";
+                    EXV_WARNING << "Failed to decode XMP metadata.\n";
 #endif
                 }
             }
@@ -362,8 +367,9 @@ namespace Exiv2 {
 
         // We look if an Adobe XMP string exist.
 
-        if ( memcmp("XML:com.adobe.xmp", key, 17) == 0 &&
-             pImage->xmpData().empty())
+        if (   keySize >= 17
+            && memcmp("XML:com.adobe.xmp", key, 17) == 0
+            && pImage->xmpData().empty())
         {
             if (arr.size_ > 0)
             {
@@ -373,15 +379,15 @@ namespace Exiv2 {
                 if (idx != std::string::npos && idx > 0)
                 {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Removing " << idx << " characters "
-                              << "from the beginning of the XMP packet\n";
+                    EXV_WARNING << "Removing " << idx << " characters "
+                                << "from the beginning of the XMP packet\n";
 #endif
                     xmpPacket = xmpPacket.substr(idx);
                 }
                 if (XmpParser::decode(pImage->xmpData(), xmpPacket))
                 {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to decode XMP metadata.\n";
+                    EXV_WARNING << "Failed to decode XMP metadata.\n";
 #endif
                 }
             }
@@ -390,8 +396,9 @@ namespace Exiv2 {
         // We look if a comments string exist. Note than we use only 'Description' keyword which
         // is dedicaced to store long comments. 'Comment' keyword is ignored.
 
-        if ( memcmp("Description", key, 11) == 0 &&
-             pImage->comment().empty())
+        if (   keySize >= 11
+            && memcmp("Description", key, 11) == 0
+            && pImage->comment().empty())
         {
             pImage->comment().assign(reinterpret_cast<char*>(arr.pData_), arr.size_);
         }
@@ -433,53 +440,48 @@ namespace Exiv2 {
     {
         uLongf uncompressedLen = compressedTextSize * 2; // just a starting point
         int zlibResult;
+        int dos = 0;
 
-        do
-        {
+        do {
             arr.alloc(uncompressedLen);
-            zlibResult = uncompress((Bytef*)arr.pData_, &uncompressedLen,
-                                    compressedText, compressedTextSize);
-
-            if (zlibResult == Z_OK)
-            {
-                // then it is all OK
+            zlibResult = uncompress((Bytef*)arr.pData_,
+                                    &uncompressedLen,
+                                    compressedText,
+                                    compressedTextSize);
+            if (zlibResult == Z_OK) {
                 arr.alloc(uncompressedLen);
             }
-            else if (zlibResult == Z_BUF_ERROR)
-            {
+            else if (zlibResult == Z_BUF_ERROR) {
                 // the uncompressedArray needs to be larger
-#ifdef DEBUG
-                std::cout << "Exiv2::PngChunk::parsePngChunk: doubling size for decompression.\n";
-#endif
                 uncompressedLen *= 2;
-
                 // DoS protection. can't be bigger than 64k
-                if ( uncompressedLen > 131072 )
-                    break;
+                if (uncompressedLen > 131072) {
+                    if (++dos > 1) break;
+                    uncompressedLen = 131072;
+                }
             }
-            else
-            {
+            else {
                 // something bad happened
                 throw Error(14);
             }
         }
         while (zlibResult == Z_BUF_ERROR);
 
-        if (zlibResult != Z_OK)
+        if (zlibResult != Z_OK) {
             throw Error(14);
-
+        }
     } // PngChunk::zlibUncompress
 
     std::string PngChunk::zlibCompress(const std::string& text)
     {
-        uLongf compressedLen = text.size() * 2; // just a starting point
+        uLongf compressedLen = static_cast<uLongf>(text.size() * 2); // just a starting point
         int zlibResult;
 
         DataBuf arr;
         do {
             arr.alloc(compressedLen);
             zlibResult = compress2((Bytef*)arr.pData_, &compressedLen,
-                                   (const Bytef*)text.data(), text.size(),
+                                   (const Bytef*)text.data(), static_cast<uLong>(text.size()),
                                    Z_BEST_COMPRESSION);
 
             switch (zlibResult) {
@@ -533,11 +535,11 @@ namespace Exiv2 {
         }
         // Determine length of the chunk data
         byte length[4];
-        ul2Data(length, chunkData.size(), bigEndian);
+        ul2Data(length, static_cast<uint32_t>(chunkData.size()), bigEndian);
         // Calculate CRC on chunk type and chunk data
         std::string crcData = chunkType + chunkData;
         uLong tmp = crc32(0L, Z_NULL, 0);
-        tmp       = crc32(tmp, (const Bytef*)crcData.data(), crcData.size());
+        tmp       = crc32(tmp, (const Bytef*)crcData.data(), static_cast<uInt>(crcData.size()));
         byte crc[4];
         ul2Data(crc, tmp, bigEndian);
         // Assemble the chunk
@@ -569,12 +571,12 @@ namespace Exiv2 {
         }
         // Determine length of the chunk data
         byte length[4];
-        ul2Data(length, chunkData.size(), bigEndian);
+        ul2Data(length, static_cast<uint32_t>(chunkData.size()), bigEndian);
         // Calculate CRC on chunk type and chunk data
         std::string chunkType = "iTXt";
         std::string crcData = chunkType + chunkData;
         uLong tmp = crc32(0L, Z_NULL, 0);
-        tmp       = crc32(tmp, (const Bytef*)crcData.data(), crcData.size());
+        tmp       = crc32(tmp, (const Bytef*)crcData.data(), static_cast<uInt>(crcData.size()));
         byte crc[4];
         ul2Data(crc, tmp, bigEndian);
         // Assemble the chunk
