@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      preview.cpp
-  Version:   $Rev: 1750 $
+  Version:   $Rev: 2455 $
   Author(s): Vladimir Nadvornik (vn) <nadvornik@suse.cz>
   History:   18-Sep-08, vn: created
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: preview.cpp 1750 2009-02-16 14:30:51Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: preview.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -218,7 +218,7 @@ namespace {
         //! Tag that contains image data. Possible values are "StripOffsets" or "TileOffsets"
         std::string offsetTag_;
 
-        //! Tag that contains data sizes. Possible values are "StripByteCountss" or "TileByteCounts"
+        //! Tag that contains data sizes. Possible values are "StripByteCounts" or "TileByteCounts"
         std::string sizeTag_;
 
         //! Structure that lists preview groups
@@ -247,21 +247,25 @@ namespace {
         { 0,                   createLoaderExifDataJpeg, 4 },
         { 0,                   createLoaderExifDataJpeg, 5 },
         { 0,                   createLoaderExifDataJpeg, 6 },
-        { "image/x-raw",       createLoaderExifDataJpeg, 7 },
+        { 0,                   createLoaderExifDataJpeg, 7 },
+        { "image/x-panasonic-rw2", createLoaderExifDataJpeg, 8 },
+        { 0,                   createLoaderExifDataJpeg, 9 },
         { 0,                   createLoaderTiff,         0 },
         { 0,                   createLoaderTiff,         1 },
         { 0,                   createLoaderTiff,         2 },
         { 0,                   createLoaderTiff,         3 },
         { 0,                   createLoaderTiff,         4 },
         { 0,                   createLoaderTiff,         5 },
+        { 0,                   createLoaderTiff,         6 },
         { 0,                   createLoaderExifJpeg,     0 },
         { 0,                   createLoaderExifJpeg,     1 },
         { 0,                   createLoaderExifJpeg,     2 },
         { 0,                   createLoaderExifJpeg,     3 },
         { 0,                   createLoaderExifJpeg,     4 },
         { 0,                   createLoaderExifJpeg,     5 },
-        { "image/x-canon-cr2", createLoaderExifJpeg,     6 },
-        { 0,                   createLoaderExifJpeg,     7 }
+        { 0,                   createLoaderExifJpeg,     6 },
+        { "image/x-canon-cr2", createLoaderExifJpeg,     7 },
+        { 0,                   createLoaderExifJpeg,     8 }
     };
 
     const LoaderExifJpeg::Param LoaderExifJpeg::param_[] = {
@@ -270,9 +274,10 @@ namespace {
         { "Exif.SubImage2.JPEGInterchangeFormat", "Exif.SubImage2.JPEGInterchangeFormatLength", 0 }, // 2
         { "Exif.SubImage3.JPEGInterchangeFormat", "Exif.SubImage3.JPEGInterchangeFormatLength", 0 }, // 3
         { "Exif.SubImage4.JPEGInterchangeFormat", "Exif.SubImage4.JPEGInterchangeFormatLength", 0 }, // 4
-        { "Exif.Image2.JPEGInterchangeFormat",    "Exif.Image2.JPEGInterchangeFormatLength",    0 }, // 5
-        { "Exif.Image.StripOffsets",              "Exif.Image.StripByteCounts",                 0 }, // 6
-        { "Exif.OlympusCs.PreviewImageStart",     "Exif.OlympusCs.PreviewImageLength",            "Exif.MakerNote.Offset"}  // 7
+        { "Exif.SubThumb1.JPEGInterchangeFormat", "Exif.SubThumb1.JPEGInterchangeFormatLength", 0 }, // 5
+        { "Exif.Image2.JPEGInterchangeFormat",    "Exif.Image2.JPEGInterchangeFormatLength",    0 }, // 6
+        { "Exif.Image.StripOffsets",              "Exif.Image.StripByteCounts",                 0 }, // 7
+        { "Exif.OlympusCs.PreviewImageStart",     "Exif.OlympusCs.PreviewImageLength",          "Exif.MakerNote.Offset"}  // 8
     };
 
     const LoaderExifDataJpeg::Param LoaderExifDataJpeg::param_[] = {
@@ -280,10 +285,12 @@ namespace {
         { "Exif.NikonPreview.JPEGInterchangeFormat", "Exif.NikonPreview.JPEGInterchangeFormatLength" }, // 1
         { "Exif.Pentax.PreviewOffset",               "Exif.Pentax.PreviewLength"                     }, // 2
         { "Exif.Minolta.ThumbnailOffset",            "Exif.Minolta.ThumbnailLength"                  }, // 3
-        { "Exif.Olympus.ThumbnailImage",             0                                               }, // 4
-        { "Exif.Olympus2.ThumbnailImage",            0                                               }, // 5
-        { "Exif.Minolta.Thumbnail",                  0                                               }, // 6
-        { "Exif.PanasonicRaw.PreviewImage",          0                                               }  // 7
+        { "Exif.SonyMinolta.ThumbnailOffset",        "Exif.SonyMinolta.ThumbnailLength"              }, // 4
+        { "Exif.Olympus.ThumbnailImage",             0                                               }, // 5
+        { "Exif.Olympus2.ThumbnailImage",            0                                               }, // 6
+        { "Exif.Minolta.Thumbnail",                  0                                               }, // 7
+        { "Exif.PanasonicRaw.PreviewImage",          0                                               }, // 8
+        { "Exif.SamsungPreview.JPEGInterchangeFormat", "Exif.SamsungPreview.JPEGInterchangeFormatLength" } // 9
     };
 
     const LoaderTiff::Param LoaderTiff::param_[] = {
@@ -292,7 +299,8 @@ namespace {
         { "SubImage2", "Exif.SubImage2.NewSubfileType", "1" },  // 2
         { "SubImage3", "Exif.SubImage3.NewSubfileType", "1" },  // 3
         { "SubImage4", "Exif.SubImage4.NewSubfileType", "1" },  // 4
-        { "Thumbnail", 0,                               0   }   // 5
+        { "SubThumb1", "Exif.SubThumb1.NewSubfileType", "1" },  // 5
+        { "Thumbnail", 0,                               0   }   // 6
     };
 
     Loader::AutoPtr Loader::create(PreviewId id, const Image &image)
@@ -338,13 +346,13 @@ namespace {
     {
         offset_ = 0;
         ExifData::const_iterator pos = image_.exifData().findKey(ExifKey(param_[parIdx].offsetKey_));
-        if (pos != image_.exifData().end()) {
+        if (pos != image_.exifData().end() && pos->count() > 0) {
             offset_ = pos->toLong();
         }
 
         size_ = 0;
         pos = image_.exifData().findKey(ExifKey(param_[parIdx].sizeKey_));
-        if (pos != image_.exifData().end()) {
+        if (pos != image_.exifData().end() && pos->count() > 0) {
             size_ = pos->toLong();
         }
 
@@ -352,7 +360,7 @@ namespace {
 
         if (param_[parIdx].baseOffsetKey_) {
             pos = image_.exifData().findKey(ExifKey(param_[parIdx].baseOffsetKey_));
-            if (pos != image_.exifData().end()) {
+            if (pos != image_.exifData().end() && pos->count() > 0) {
                 offset_ += pos->toLong();
             }
         }
@@ -372,6 +380,9 @@ namespace {
         PreviewProperties prop = Loader::getProperties();
         prop.mimeType_ = "image/jpeg";
         prop.extension_ = ".jpg";
+#ifdef EXV_UNICODE_PATH
+        prop.wextension_ = EXV_WIDEN(".jpg");
+#endif
         return prop;
     }
 
@@ -385,7 +396,7 @@ namespace {
         }
         IoCloser closer(io);
 
-        const byte *base = io.mmap();
+        const Exiv2::byte* base = io.mmap();
 
         return DataBuf(base + offset_, size_);
     }
@@ -401,7 +412,7 @@ namespace {
             throw Error(9, io.path(), strError());
         }
         IoCloser closer(io);
-        const byte *base = io.mmap();
+        const Exiv2::byte* base = io.mmap();
 
         try {
             Image::AutoPtr image = ImageFactory::open(base + offset_, size_);
@@ -413,7 +424,7 @@ namespace {
         }
         catch (const AnyError& /* error */ ) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: Invalid JPEG preview image.\n";
+            EXV_WARNING << "Invalid JPEG preview image.\n";
 #endif
             return false;
         }
@@ -447,6 +458,9 @@ namespace {
         PreviewProperties prop = Loader::getProperties();
         prop.mimeType_ = "image/jpeg";
         prop.extension_ = ".jpg";
+#ifdef EXV_UNICODE_PATH
+        prop.wextension_ = EXV_WIDEN(".jpg");
+#endif
         return prop;
     }
 
@@ -526,19 +540,19 @@ namespace {
         if (pos == exifData.end()) return;
         if (offsetCount != pos->value().count()) return;
         for (int i = 0; i < offsetCount; i++) {
-            size_ += pos->value().toLong(i);
+            size_ += pos->toLong(i);
         }
 
         if (size_ == 0) return;
 
         pos = exifData.findKey(ExifKey(std::string("Exif.") + group_ + ".ImageWidth"));
-        if (pos != exifData.end()) {
-            width_ = pos->value().toLong();
+        if (pos != exifData.end() && pos->count() > 0) {
+            width_ = pos->toLong();
         }
 
         pos = exifData.findKey(ExifKey(std::string("Exif.") + group_ + ".ImageLength"));
-        if (pos != exifData.end()) {
-            height_ = pos->value().toLong();
+        if (pos != exifData.end() && pos->count() > 0) {
+            height_ = pos->toLong();
         }
 
         if (width_ == 0 || height_ == 0) return;
@@ -556,6 +570,9 @@ namespace {
         PreviewProperties prop = Loader::getProperties();
         prop.mimeType_ = "image/tiff";
         prop.extension_ = ".tif";
+#ifdef EXV_UNICODE_PATH
+        prop.wextension_ = EXV_WIDEN(".tif");
+#endif
         return prop;
     }
 
@@ -610,38 +627,40 @@ namespace {
             }
             IoCloser closer(io);
 
-            const byte *base = io.mmap();
+            const Exiv2::byte* base = io.mmap();
 
             const Value &sizes = preview["Exif.Image." + sizeTag_].value();
 
-            if (sizes.count() == 1) {
-                // this saves one copying of the buffer
-                uint32_t offset = dataValue.toLong(0);
-                uint32_t size = sizes.toLong(0);
-                if (offset + size <= static_cast<uint32_t>(io.size()))
-                    dataValue.setDataArea(base + offset, size);
-            }
-            else {
-                // FIXME: the buffer is probably copied twice, it should be optimized
-                DataBuf buf(size_);
-                byte *pos = buf.pData_;
-                for (int i = 0; i < sizes.count(); i++) {
-                    uint32_t offset = dataValue.toLong(i);
-                    uint32_t size = sizes.toLong(i);
+            if (sizes.count() == dataValue.count()) {
+                if (sizes.count() == 1) {
+                    // this saves one copying of the buffer
+                    uint32_t offset = dataValue.toLong(0);
+                    uint32_t size = sizes.toLong(0);
                     if (offset + size <= static_cast<uint32_t>(io.size()))
-                        memcpy(pos, base + offset, size);
-                    pos += size;
+                        dataValue.setDataArea(base + offset, size);
+                }
+                else {
+                    // FIXME: the buffer is probably copied twice, it should be optimized
+                    DataBuf buf(size_);
+                    Exiv2::byte* pos = buf.pData_;
+                    for (int i = 0; i < sizes.count(); i++) {
+                        uint32_t offset = dataValue.toLong(i);
+                        uint32_t size = sizes.toLong(i);
+                        if (offset + size <= static_cast<uint32_t>(io.size()))
+                            memcpy(pos, base + offset, size);
+                        pos += size;
+                    }
+                    dataValue.setDataArea(buf.pData_, buf.size_);
                 }
-                dataValue.setDataArea(buf.pData_, buf.size_);
             }
         }
 
         // write new image
-        Blob blob;
-        const IptcData emptyIptc;
-        const XmpData  emptyXmp;
-        TiffParser::encode(blob, 0, 0, Exiv2::littleEndian, preview, emptyIptc, emptyXmp);
-        return DataBuf((blob.size() > 0 ? &blob[0] : 0), static_cast<long>(blob.size()));
+        MemIo mio;
+        IptcData emptyIptc;
+        XmpData  emptyXmp;
+        TiffParser::encode(mio, 0, 0, Exiv2::littleEndian, preview, emptyIptc, emptyXmp);
+        return DataBuf(mio.mmap(), mio.size());
     }
 
 }                                       // namespace
@@ -692,6 +711,16 @@ namespace Exiv2 {
         return Exiv2::writeFile(buf, name);
     }
 
+#ifdef EXV_UNICODE_PATH
+    long PreviewImage::writeFile(const std::wstring& wpath) const
+    {
+        std::wstring name = wpath + wextension();
+        // Todo: Creating a DataBuf here unnecessarily copies the memory
+        DataBuf buf(pData_, size_);
+        return Exiv2::writeFile(buf, name);
+    }
+
+#endif
     DataBuf PreviewImage::copy() const
     {
         return DataBuf(pData_, size_);
@@ -717,6 +746,13 @@ namespace Exiv2 {
         return properties_.extension_;
     }
 
+#ifdef EXV_UNICODE_PATH
+    std::wstring PreviewImage::wextension() const
+    {
+        return properties_.wextension_;
+    }
+
+#endif
     uint32_t PreviewImage::width() const
     {
         return properties_.width_;
@@ -740,8 +776,8 @@ namespace Exiv2 {
     PreviewPropertiesList PreviewManager::getPreviewProperties() const
     {
         PreviewPropertiesList list;
-        // go through the loader table and store all successfuly created loaders in the list
-        for (PreviewId id = 0; id < Loader::getNumLoaders(); id++) {
+        // go through the loader table and store all successfully created loaders in the list
+        for (PreviewId id = 0; id < Loader::getNumLoaders(); ++id) {
             Loader::AutoPtr loader = Loader::create(id, image_);
             if (loader.get() && loader->readDimensions()) {
                 list.push_back(loader->getProperties());
