@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      psdimage.cpp
-  Version:   $Rev: 1774 $
+  Version:   $Rev: 2455 $
   Author(s): Marco Piovanelli, Ovolab (marco)
              Michael Ulbrich (mul)
   History:   05-Mar-2007, marco: created
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: psdimage.cpp 1774 2009-03-28 11:18:55Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: psdimage.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 //#define DEBUG 1
 
@@ -137,6 +137,11 @@ namespace Exiv2 {
     {
     } // PsdImage::PsdImage
 
+    std::string PsdImage::mimeType() const
+    {
+        return "image/x-photoshop";
+    }
+
     void PsdImage::setComment(const std::string& /*comment*/)
     {
         // not supported
@@ -254,7 +259,7 @@ namespace Exiv2 {
                 if (io_->error() || io_->eof()) throw Error(14);
                 if (IptcParser::decode(iptcData_, rawIPTC.pData_, rawIPTC.size_)) {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to decode IPTC metadata.\n";
+                    EXV_WARNING << "Failed to decode IPTC metadata.\n";
 #endif
                     iptcData_.clear();
                 }
@@ -270,7 +275,7 @@ namespace Exiv2 {
                 setByteOrder(bo);
                 if (rawExif.size_ > 0 && byteOrder() == invalidByteOrder) {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to decode Exif metadata.\n";
+                    EXV_WARNING << "Failed to decode Exif metadata.\n";
 #endif
                     exifData_.clear();
                 }
@@ -285,7 +290,7 @@ namespace Exiv2 {
                 xmpPacket_.assign(reinterpret_cast<char *>(xmpPacket.pData_), xmpPacket.size_);
                 if (xmpPacket_.size() > 0 && XmpParser::decode(xmpData_, xmpPacket_)) {
 #ifndef SUPPRESS_WARNINGS
-                    std::cerr << "Warning: Failed to decode XMP metadata.\n";
+                    EXV_WARNING << "Failed to decode XMP metadata.\n";
 #endif
                 }
                 break;
@@ -342,7 +347,7 @@ namespace Exiv2 {
         // Write Photoshop header data out to new PSD file
         if (outIo.write(psd_head, 26) != 26) throw Error(21);
 
-        // Read colorDataLength from original PSD 
+        // Read colorDataLength from original PSD
         if (io_->read(buf, 4) != 4) throw Error(3, "Photoshop");
 
         uint32_t colorDataLength = getULong(buf, bigEndian);
@@ -358,7 +363,7 @@ namespace Exiv2 {
         long toRead = 0;
         while (readTotal < colorDataLength) {
             toRead =   static_cast<long>(colorDataLength - readTotal) < lbuf.size_
-                     ? colorDataLength - readTotal : lbuf.size_;
+                     ? static_cast<long>(colorDataLength - readTotal) : lbuf.size_;
             if (io_->read(lbuf.pData_, toRead) != toRead) throw Error(3, "Photoshop");
             readTotal += toRead;
             if (outIo.write(lbuf.pData_, toRead) != toRead) throw Error(21);
@@ -408,30 +413,30 @@ namespace Exiv2 {
                 != static_cast<long>(adjResourceNameLen)) throw Error(3, "Photoshop");
 
             // read resource size (actual length w/o padding!)
-            if (io_->read(buf, 4) != 4) throw Error(3, "Photoshop"); 
+            if (io_->read(buf, 4) != 4) throw Error(3, "Photoshop");
 
             uint32_t resourceSize = getULong(buf, bigEndian);
             uint32_t curOffset = io_->tell();
 
             // Write IPTC_NAA resource block
-            if (   resourceId == kPhotoshopResourceID_IPTC_NAA
-                || (resourceId > kPhotoshopResourceID_IPTC_NAA && iptcDone == false)) {
+            if ((resourceId == kPhotoshopResourceID_IPTC_NAA  ||
+                 resourceId >  kPhotoshopResourceID_IPTC_NAA) && iptcDone == false) {
                 newResLength += writeIptcData(iptcData_, outIo);
                 resourceSize = (resourceSize + 1) & ~1;    // adjust for padding
                 iptcDone = true;
             }
 
             // Write ExifInfo resource block
-            else if (   resourceId == kPhotoshopResourceID_ExifInfo
-                     || (resourceId > kPhotoshopResourceID_ExifInfo && exifDone == false)) {
+            else if ((resourceId == kPhotoshopResourceID_ExifInfo  ||
+                      resourceId >  kPhotoshopResourceID_ExifInfo) && exifDone == false) {
                 newResLength += writeExifData(exifData_, outIo);
                 resourceSize = (resourceSize + 1) & ~1;    // adjust for padding
                 exifDone = true;
             }
 
             // Write XMPpacket resource block
-            else if (   resourceId == kPhotoshopResourceID_XMPPacket
-                     || (resourceId > kPhotoshopResourceID_XMPPacket && xmpDone == false)) {
+            else if ((resourceId == kPhotoshopResourceID_XMPPacket  ||
+                      resourceId >  kPhotoshopResourceID_XMPPacket) && xmpDone == false) {
                 newResLength += writeXmpData(xmpData_, outIo);
                 resourceSize = (resourceSize + 1) & ~1;    // adjust for padding
                 xmpDone = true;
@@ -451,7 +456,7 @@ namespace Exiv2 {
                 us2Data(buf, resourceId, bigEndian);
                 if (outIo.write(buf, 2) != 2) throw Error(21);
                 // Write resource name as Pascal string
-                buf[0] = resourceNameLength & 0x000f;
+                buf[0] = resourceNameLength & 0x00ff;
                 if (outIo.write(buf, 1) != 1) throw Error(21);
                 buf[0] = resourceNameFirstChar;
                 if (outIo.write(buf, 1) != 1) throw Error(21);
@@ -465,7 +470,7 @@ namespace Exiv2 {
                 resourceSize = (resourceSize + 1) & ~1;        // pad to even
                 while (readTotal < resourceSize) {
                     toRead =   static_cast<long>(resourceSize - readTotal) < lbuf.size_
-                             ? resourceSize - readTotal : lbuf.size_;
+                             ? static_cast<long>(resourceSize - readTotal) : lbuf.size_;
                     if (io_->read(lbuf.pData_, toRead) != toRead) {
                         throw Error(3, "Photoshop");
                     }
@@ -574,11 +579,11 @@ namespace Exiv2 {
                 if (out.write(buf, 2) != 2) throw Error(21);
                 us2Data(buf, 0, bigEndian);                      // NULL resource name
                 if (out.write(buf, 2) != 2) throw Error(21);
-                ul2Data(buf, blob.size(), bigEndian);
+                ul2Data(buf, static_cast<uint32_t>(blob.size()), bigEndian);
                 if (out.write(buf, 4) != 4) throw Error(21);
                 // Write encoded Exif data
-                if (out.write(&blob[0], blob.size()) != static_cast<long>(blob.size())) throw Error(21);
-                resLength += blob.size() + 12;
+                if (out.write(&blob[0], static_cast<long>(blob.size())) != static_cast<long>(blob.size())) throw Error(21);
+                resLength += static_cast<long>(blob.size()) + 12;
                 if (blob.size() & 1)    // even padding
                 {
                     buf[0] = 0;
@@ -603,7 +608,7 @@ namespace Exiv2 {
         if (writeXmpFromPacket() == false) {
             if (XmpParser::encode(xmpPacket, xmpData) > 1) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Error: Failed to encode XMP metadata.\n";
+                EXV_ERROR << "Failed to encode XMP metadata.\n";
 #endif
             }
         }
@@ -619,13 +624,13 @@ namespace Exiv2 {
             if (out.write(buf, 2) != 2) throw Error(21);
             us2Data(buf, 0, bigEndian);                      // NULL resource name
             if (out.write(buf, 2) != 2) throw Error(21);
-            ul2Data(buf, xmpPacket.size(), bigEndian);
+            ul2Data(buf, static_cast<uint32_t>(xmpPacket.size()), bigEndian);
             if (out.write(buf, 4) != 4) throw Error(21);
             // Write XMPPacket
             if (out.write(reinterpret_cast<const byte*>(xmpPacket.data()), static_cast<long>(xmpPacket.size()))
                 != static_cast<long>(xmpPacket.size())) throw Error(21);
             if (out.error()) throw Error(21);
-            resLength += xmpPacket.size() + 12;
+            resLength += static_cast<uint32_t>(xmpPacket.size()) + 12;
             if (xmpPacket.size() & 1)    // even padding
             {
                 buf[0] = 0;
