@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tiffimage_int.hpp
   @brief   Internal class TiffParserWorker to parse TIFF data.
-  @version $Rev: 1803 $
+  @version $Rev: 2455 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    23-Apr-08, ahu: created
@@ -32,7 +32,9 @@
 // *****************************************************************************
 // included header files
 #include "tifffwd_int.hpp"
+#include "tiffcomposite_int.hpp"
 #include "image.hpp"
+#include "tags_int.hpp"
 #include "types.hpp"
 
 // + standard includes
@@ -89,13 +91,12 @@ namespace Exiv2 {
         //! @name Accessors
         //@{
         /*!
-          @brief Write the image header to the binary image \em blob.
-                 This method appends to the blob.
+          @brief Return the image header in binary format.
+                 The caller owns this data and %DataBuf ensures that it will be deleted.
 
-          @param blob Binary image to add to.
-          @return Number of bytes written.
+          @return Binary header data.
          */
-        virtual uint32_t write(Blob& blob) const;
+        virtual DataBuf write() const;
         /*!
           @brief Print debug info for the image header to \em os.
 
@@ -109,8 +110,25 @@ namespace Exiv2 {
         virtual uint32_t offset() const;
         //! Return the size (in bytes) of the image header.
         virtual uint32_t size() const;
-        //! Return the tag value (magic number) which identifies the buffer as TIFF data
+        //! Return the tag value (magic number) which identifies the buffer as TIFF data.
         virtual uint16_t tag() const;
+        /*!
+          @brief Return \c true if the %Exif \em tag from \em group is an image tag.
+
+          Certain tags of TIFF and TIFF-like images are required to correctly
+          display the primary image. These image tags contain image data rather
+          than metadata.
+
+          @param tag Tag number.
+          @param group Group identifier.
+          @param pPrimaryGroups Pointer to a list of TIFF groups that contain
+                 primary images, empty if none are marked.
+
+          @return The default implementation returns \c false.
+         */
+        virtual bool isImageTag(      uint16_t       tag,
+                                      IfdId          group,
+                                const PrimaryGroups* pPrimaryGroups) const;
         //@}
 
     private:
@@ -130,38 +148,75 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         //! Default constructor
-        TiffHeader(ByteOrder byteOrder =littleEndian,
-                   uint32_t  offset    =0x00000008);
+        TiffHeader(ByteOrder byteOrder    =littleEndian,
+                   uint32_t  offset       =0x00000008,
+                   bool      hasImageTags =true);
         //! Destructor
         ~TiffHeader();
         //@}
+        //@{
+        //! @name Accessors
+        bool isImageTag(      uint16_t       tag,
+                              IfdId          group,
+                        const PrimaryGroups* pPrimaryGroups) const;
+        //@}
+
+    private:
+        // DATA
+        bool           hasImageTags_;   //!< Indicates if image tags are supported
     }; // class TiffHeader
 
+    /*!
+      @brief Data structure used to list image tags for TIFF and TIFF-like images.
+     */
+    struct TiffImgTagStruct {
+        //! Search key for TIFF image tag structure.
+        struct Key {
+            //! Constructor
+            Key(uint16_t t, IfdId g) : t_(t), g_(g) {}
+            uint16_t t_;                    //!< %Tag
+            IfdId    g_;                    //!< %Group
+        };
+
+        //! Comparison operator to compare a TiffImgTagStruct with a TiffImgTagStruct::Key
+        bool operator==(const Key& key) const
+        {
+            return key.g_ == group_ && key.t_ == tag_;
+        }
+
+        // DATA
+        uint16_t       tag_;            //!< Image tag
+        IfdId          group_;          //!< Group that contains the image tag
+    }; // struct TiffImgTagStruct
+
     /*!
       @brief Data structure used as a row (element) of a table (array)
              defining the TIFF component used for each tag in a group.
      */
     struct TiffGroupStruct {
-        struct Key;
+        //! Search key for TIFF group structure.
+        struct Key {
+            //! Constructor
+            Key(uint32_t e, IfdId g) : e_(e), g_(g) {}
+            uint32_t e_;                    //!< Extended tag
+            IfdId    g_;                    //!< %Group
+        };
+
         //! Comparison operator to compare a TiffGroupStruct with a TiffGroupStruct::Key
-        bool operator==(const Key& key) const;
+        bool operator==(const Key& key) const
+        {
+            return    key.g_ == group_
+                   && (Tag::all == extendedTag_ || key.e_ == extendedTag_);
+        }
         //! Return the tag corresponding to the extended tag
         uint16_t tag() const { return static_cast<uint16_t>(extendedTag_ & 0xffff); }
 
         // DATA
         uint32_t       extendedTag_;    //!< Tag (32 bit so that it can contain special tags)
-        uint16_t       group_;          //!< Group that contains the tag
+        IfdId          group_;          //!< Group that contains the tag
         NewTiffCompFct newTiffCompFct_; //!< Function to create the correct TIFF component
     };
 
-    //! Search key for TIFF group structure.
-    struct TiffGroupStruct::Key {
-        //! Constructor
-        Key(uint32_t e, uint16_t g) : e_(e), g_(g) {}
-        uint32_t e_;                    //!< Extended tag
-        uint16_t g_;                    //!< %Group
-    };
-
     /*!
       @brief Data structure used as a row of the table which describes TIFF trees.
              Multiple trees are needed as TIFF-based RAW image formats do not always
@@ -174,17 +229,17 @@ namespace Exiv2 {
 
         // DATA
         uint32_t       root_;           //!< Tree root element, identifies a tree
-        uint16_t       group_;          //!< Each group is a node in the tree
-        uint16_t       parentGroup_;    //!< Parent group
+        IfdId          group_;          //!< Each group is a node in the tree
+        IfdId          parentGroup_;    //!< Parent group
         uint32_t       parentExtTag_;   //!< Parent tag (32 bit so that it can contain special tags)
     };
 
     //! Search key for TIFF tree structure.
     struct TiffTreeStruct::Key {
         //! Constructor
-        Key(uint32_t r, uint16_t g) : r_(r), g_(g) {}
+        Key(uint32_t r, IfdId g) : r_(r), g_(g) {}
         uint32_t r_;                    //!< Root
-        uint16_t g_;                    //!< %Group
+        IfdId    g_;                    //!< %Group
     };
 
     /*!
@@ -199,7 +254,7 @@ namespace Exiv2 {
                  is 0, then the TIFF entry should be ignored.
         */
         static std::auto_ptr<TiffComponent> create(uint32_t extendedTag,
-                                                   uint16_t group);
+                                                   IfdId    group);
         /*!
           @brief Get the path, i.e., a list of extended tag and group pairs, from
                  the \em root TIFF element to the TIFF entry \em extendedTag and
@@ -207,7 +262,7 @@ namespace Exiv2 {
         */
         static void getPath(TiffPath& tiffPath,
                             uint32_t  extendedTag,
-                            uint16_t  group,
+                            IfdId     group,
                             uint32_t  root);
 
     private:
@@ -252,13 +307,20 @@ namespace Exiv2 {
                   uint32_t           size,
                   uint32_t           root,
                   FindDecoderFct     findDecoderFct,
-                  TiffHeaderBase*    pHeader =0);
+                  TiffHeaderBase*    pHeader =0
+        );
         /*!
           @brief Encode TIFF metadata from the metadata containers into a
                  memory block \em blob.
-        */
+
+          1) Parse the binary image, if one is provided, and
+          2) attempt updating the parsed tree in-place ("non-intrusive writing")
+          3) else, create a new tree and write a new TIFF structure ("intrusive
+             writing"). If there is a parsed tree, it is only used to access the
+             image data in this case.
+         */
         static WriteMethod encode(
-                  Blob&              blob,
+                  BasicIo&           io,
             const byte*              pData,
                   uint32_t           size,
             const ExifData&          exifData,
@@ -283,11 +345,23 @@ namespace Exiv2 {
                            composite structure. If \em pData is 0 or \em size
                            is 0, the return value is a 0 pointer.
          */
-        static std::auto_ptr<TiffComponent>
-        parse(const byte*              pData,
-                    uint32_t           size,
-                    uint32_t           root,
-                    TiffHeaderBase*    pHeader);
+        static std::auto_ptr<TiffComponent> parse(
+            const byte*              pData,
+                  uint32_t           size,
+                  uint32_t           root,
+                  TiffHeaderBase*    pHeader
+        );
+        /*!
+          @brief Find primary groups in the source tree provided and populate
+                 the list of primary groups.
+
+          @param primaryGroups List of primary groups which is populated
+          @param pSourceDir Pointer to the source composite tree to search (may be 0)
+         */
+        static void findPrimaryGroups(
+            PrimaryGroups& primaryGroups,
+            TiffComponent* pSourceDir
+        );
 
     }; // class TiffParserWorker
 
@@ -314,7 +388,7 @@ namespace Exiv2 {
          */
         static DecoderFct findDecoder(const std::string& make,
                                             uint32_t     extendedTag,
-                                            uint16_t     group);
+                                            IfdId        group);
         /*!
           @brief Find special encoder function for a key.
 
@@ -331,7 +405,7 @@ namespace Exiv2 {
         static EncoderFct findEncoder(
             const std::string& make,
                   uint32_t     extendedTag,
-                  uint16_t     group
+                  IfdId        group
         );
 
     private:
@@ -339,16 +413,17 @@ namespace Exiv2 {
 
     }; // class TiffMapping
 
+    // Todo: Move this class to metadatum_int.hpp or tags_int.hpp
     //! Unary predicate that matches an Exifdatum with a given IfdId.
     class FindExifdatum {
     public:
         //! Constructor, initializes the object with the IfdId to look for.
-        FindExifdatum(Exiv2::IfdId ifdId) : ifdId_(ifdId) {}
+        FindExifdatum(Exiv2::Internal::IfdId ifdId) : ifdId_(ifdId) {}
         //! Returns true if IFD id matches.
         bool operator()(const Exiv2::Exifdatum& md) const { return ifdId_ == md.ifdId(); }
 
     private:
-        Exiv2::IfdId ifdId_;
+        Exiv2::Internal::IfdId ifdId_;
 
     }; // class FindExifdatum
 
