@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -22,7 +22,7 @@
   @file    tiffvisitor_int.hpp
   @brief   Internal operations on a TIFF composite tree, implemented as visitor
            classes.
-  @version $Rev: 1773 $
+  @version $Rev: 2455 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Apr-06, ahu: created
@@ -43,10 +43,15 @@
 #include <cassert>
 #include <map>
 #include <set>
+#include <vector>
 
 // *****************************************************************************
 // namespace extensions
 namespace Exiv2 {
+
+    class IptcData;
+    class XmpData;
+
     namespace Internal {
 
 // *****************************************************************************
@@ -88,7 +93,7 @@ namespace Exiv2 {
         //! Default constructor. Initialises all stop/go flags to true.
         TiffVisitor();
         //! Virtual destructor
-        virtual ~TiffVisitor() {}
+        virtual ~TiffVisitor();
         //@}
 
         //! @name Manipulators
@@ -119,12 +124,12 @@ namespace Exiv2 {
           @brief Operation to perform for a TIFF directory, after all components
                  and before the next entry is processed.
          */
-        virtual void visitDirectoryNext(TiffDirectory* /*object*/) {}
+        virtual void visitDirectoryNext(TiffDirectory* object);
         /*!
           @brief Operation to perform for a TIFF directory, at the end of the
                  processing.
          */
-        virtual void visitDirectoryEnd(TiffDirectory* /*object*/) {}
+        virtual void visitDirectoryEnd(TiffDirectory* object);
         //! Operation to perform for a TIFF sub-IFD
         virtual void visitSubIfd(TiffSubIfd* object) =0;
         //! Operation to perform for the makernote component
@@ -132,11 +137,16 @@ namespace Exiv2 {
         //! Operation to perform for an IFD makernote
         virtual void visitIfdMakernote(TiffIfdMakernote* object) =0;
         //! Operation to perform after processing an IFD makernote
-        virtual void visitIfdMakernoteEnd(TiffIfdMakernote* /*object*/) {}
-        //! Operation to perform for an array entry (as found in Canon makernotes)
-        virtual void visitArrayEntry(TiffArrayEntry* object) =0;
-        //! Operation to perform for an array element
-        virtual void visitArrayElement(TiffArrayElement* object) =0;
+        virtual void visitIfdMakernoteEnd(TiffIfdMakernote* object);
+        //! Operation to perform for a binary array
+        virtual void visitBinaryArray(TiffBinaryArray* object) =0;
+        /*!
+          @brief Operation to perform for a TIFF binary array, at the end
+                 of the processing.
+         */
+        virtual void visitBinaryArrayEnd(TiffBinaryArray* object);
+        //! Operation to perform for an element of a binary array
+        virtual void visitBinaryElement(TiffBinaryElement* object) =0;
         //@}
 
         //! @name Accessors
@@ -158,10 +168,10 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         //! Constructor, taking \em tag and \em group of the component to find.
-        TiffFinder(uint16_t tag, uint16_t group)
+        TiffFinder(uint16_t tag, IfdId group)
             : tag_(tag), group_(group), tiffComponent_(0) {}
         //! Virtual destructor
-        virtual ~TiffFinder() {}
+        virtual ~TiffFinder();
         //@}
 
         //! @name Manipulators
@@ -182,15 +192,15 @@ namespace Exiv2 {
         virtual void visitMnEntry(TiffMnEntry* object);
         //! Find tag and group in an IFD makernote
         virtual void visitIfdMakernote(TiffIfdMakernote* object);
-        //! Find tag and group in an array entry component
-        virtual void visitArrayEntry(TiffArrayEntry* object);
-        //! Find tag and group in an array element
-        virtual void visitArrayElement(TiffArrayElement* object);
+        //! Find tag and group in a binary array
+        virtual void visitBinaryArray(TiffBinaryArray* object);
+        //! Find tag and group in an element of a binary array
+        virtual void visitBinaryElement(TiffBinaryElement* object);
 
         //! Check if \em object matches \em tag and \em group
         void findObject(TiffComponent* object);
         //! Initialize the Finder for a new search.
-        void init(uint16_t tag, uint16_t group);
+        void init(uint16_t tag, IfdId group);
         //@}
 
         //! @name Accessors
@@ -204,10 +214,69 @@ namespace Exiv2 {
 
     private:
         uint16_t tag_;
-        uint16_t group_;
+        IfdId group_;
         TiffComponent* tiffComponent_;
     }; // class TiffFinder
 
+    /*!
+      @brief Copy all image tags from the source tree (the tree that is traversed) to a 
+             target tree, which is empty except for the root element provided in the
+             constructor.
+    */
+    class TiffCopier : public TiffVisitor {
+    public:
+        //! @name Creators
+        //@{
+        /*!
+          @brief Constructor
+
+          @param pRoot Pointer to the root element of the (empty) target tree.
+          @param root 
+          @param pHeader Pointer to the TIFF header of the source image.
+          @param pPrimaryGroups Pointer to the list of primary groups.
+         */
+        TiffCopier(      TiffComponent*  pRoot,
+                         uint32_t        root,
+                   const TiffHeaderBase* pHeader,
+                   const PrimaryGroups*  pPrimaryGroups);
+        //! Virtual destructor
+        virtual ~TiffCopier();
+        //@}
+
+        //! @name Manipulators
+        //@{
+        //! Copy a TIFF entry if it is an image tag
+        virtual void visitEntry(TiffEntry* object);
+        //! Copy a TIFF data entry if it is an image tag
+        virtual void visitDataEntry(TiffDataEntry* object);
+        //! Copy a TIFF image entry if it is an image tag
+        virtual void visitImageEntry(TiffImageEntry* object);
+        //! Copy a TIFF size entry if it is an image tag
+        virtual void visitSizeEntry(TiffSizeEntry* object);
+        //! Copy a TIFF directory if it is an image tag
+        virtual void visitDirectory(TiffDirectory* object);
+        //! Copy a TIFF sub-IFD if it is an image tag
+        virtual void visitSubIfd(TiffSubIfd* object);
+        //! Copy a TIFF makernote if it is an image tag
+        virtual void visitMnEntry(TiffMnEntry* object);
+        //! Copy an IFD makernote if it is an image tag
+        virtual void visitIfdMakernote(TiffIfdMakernote* object);
+        //! Copy a binary array if it is an image tag
+        virtual void visitBinaryArray(TiffBinaryArray* object);
+        //! Copy an element of a binary array if it is an image tag
+        virtual void visitBinaryElement(TiffBinaryElement* object);
+
+        //! Check if \em object is an image tag and if so, copy it to the target tree.
+        void copyObject(TiffComponent* object);
+        //@}
+
+    private:
+              TiffComponent*  pRoot_;
+              uint32_t        root_;
+        const TiffHeaderBase* pHeader_;
+        const PrimaryGroups*  pPrimaryGroups_;
+    }; // class TiffCopier
+
     /*!
       @brief TIFF composite visitor to decode metadata from the TIFF tree and
              add it to an Image, which is supplied in the constructor (Visitor
@@ -231,7 +300,7 @@ namespace Exiv2 {
             FindDecoderFct       findDecoderFct
         );
         //! Virtual destructor
-        virtual ~TiffDecoder() {}
+        virtual ~TiffDecoder();
         //@}
 
         //! @name Manipulators
@@ -252,10 +321,10 @@ namespace Exiv2 {
         virtual void visitMnEntry(TiffMnEntry* object);
         //! Decode an IFD makernote
         virtual void visitIfdMakernote(TiffIfdMakernote* object);
-        //! Decode an array entry component
-        virtual void visitArrayEntry(TiffArrayEntry* object);
-        //! Decode an array element
-        virtual void visitArrayElement(TiffArrayElement* object);
+        //! Decode a binary array
+        virtual void visitBinaryArray(TiffBinaryArray* object);
+        //! Decode an element of a binary array
+        virtual void visitBinaryElement(TiffBinaryElement* object);
 
         //! Entry function, determines how to decode each tag
         void decodeTiffEntry(const TiffEntryBase* object);
@@ -281,7 +350,7 @@ namespace Exiv2 {
         void getObjData(byte const*&         pData,
                         long&                size,
                         uint16_t             tag,
-                        uint16_t             group,
+                        IfdId                group,
                         const TiffEntryBase* object);
         //@}
 
@@ -326,11 +395,13 @@ namespace Exiv2 {
             const IptcData&      iptcData,
             const XmpData&       xmpData,
                   TiffComponent* pRoot,
-                  ByteOrder      byteOrder,
+            const bool           isNewImage,
+            const PrimaryGroups* pPrimaryGroups,
+            const TiffHeaderBase* pHeader,
                   FindEncoderFct findEncoderFct
         );
         //! Virtual destructor
-        virtual ~TiffEncoder() {}
+        virtual ~TiffEncoder();
         //@}
 
         //! @name Manipulators
@@ -355,10 +426,12 @@ namespace Exiv2 {
         virtual void visitIfdMakernote(TiffIfdMakernote* object);
         //! Reset encoder to its original state, undo makernote specific settings
         virtual void visitIfdMakernoteEnd(TiffIfdMakernote* object);
-        //! Encode an array entry component
-        virtual void visitArrayEntry(TiffArrayEntry* object);
-        //! Encode an array element
-        virtual void visitArrayElement(TiffArrayElement* object);
+        //! Encode a binary array
+        virtual void visitBinaryArray(TiffBinaryArray* object);
+        //! Re-encrypt binary array if necessary
+        virtual void visitBinaryArrayEnd(TiffBinaryArray* object);
+        //! Encode an element of a binary array
+        virtual void visitBinaryElement(TiffBinaryElement* object);
 
         /*!
           @brief Top level encoder function. Determines how to encode each TIFF
@@ -384,10 +457,10 @@ namespace Exiv2 {
             const Exifdatum*     datum =0
         );
 
-        //! Callback encoder function for an array element.
-        void encodeArrayElement(TiffArrayElement* object, const Exifdatum* datum);
-        //! Callback encoder function for an array entry.
-        void encodeArrayEntry(TiffArrayEntry* object, const Exifdatum* datum);
+        //! Callback encoder function for an element of a binary array.
+        void encodeBinaryElement(TiffBinaryElement* object, const Exifdatum* datum);
+        //! Callback encoder function for a binary array.
+        void encodeBinaryArray(TiffBinaryArray* object, const Exifdatum* datum);
         //! Callback encoder function for a data entry.
         void encodeDataEntry(TiffDataEntry* object, const Exifdatum* datum);
         //! Callback encoder function for a standard TIFF entry
@@ -411,8 +484,6 @@ namespace Exiv2 {
 
         //! Special encoder function to encode IPTC data to an IPTCNAA or Photoshop ImageResources tag.
         void encodeIptc(TiffEntryBase* object, const Exifdatum* datum);
-        //! Special encoder function for a standard TIFF entry using big endian byte order.
-        void encodeBigEndianEntry(TiffEntryBase* object, const Exifdatum* datum);
         /*!
           @brief Add metadata from image to the TIFF composite.
 
@@ -472,10 +543,20 @@ namespace Exiv2 {
           @brief Update a directory entry. This is called after all directory
                  entries are encoded. It takes care of type and count changes
                  and size shrinkage for non-intrusive writing.
-        */
+         */
         uint32_t updateDirEntry(byte* buf,
                                 ByteOrder byteOrder,
                                 TiffComponent* pTiffComponent) const;
+        /*!
+          @brief Check if the tag is an image tag of an existing image. Such 
+                 tags are copied from the original image and can't be modifed.
+
+                 The condition is true if there is an existing image (as
+                 opposed to a newly created TIFF image) and \em tag, \em group
+                 is considered an image tag of this image - whether or not
+                 it's actually present in the existing image doesn't matter.
+         */
+        bool isImageTag(uint16_t tag, IfdId group) const;
         //@}
 
     private:
@@ -484,7 +565,10 @@ namespace Exiv2 {
         const IptcData& iptcData_;   //!< IPTC data to encode, just a reference
         const XmpData&  xmpData_;    //!< XMP data to encode, just a reference
         bool del_;                   //!< Indicates if Exif data entries should be deleted after encoding
+        const TiffHeaderBase* pHeader_; //!< TIFF image header
         TiffComponent* pRoot_;       //!< Root element of the composite
+        const bool isNewImage_;      //!< True if the TIFF image is created from scratch
+        const PrimaryGroups* pPrimaryGroups_; //!< List of primary image groups
         TiffComponent* pSourceTree_; //!< Parsed source tree for reference
         ByteOrder byteOrder_;        //!< Byteorder for encoding
         ByteOrder origByteOrder_;    //!< Byteorder as set in the c'tor
@@ -492,6 +576,7 @@ namespace Exiv2 {
         std::string make_;           //!< Camera make, determined from the tags to encode
         bool dirty_;                 //!< Signals if any tag is deleted or allocated
         WriteMethod writeMethod_;    //!< Write method used.
+
     }; // class TiffEncoder
 
     /*!
@@ -588,10 +673,10 @@ namespace Exiv2 {
         virtual void visitIfdMakernote(TiffIfdMakernote* object);
         //! Reset reader to its original state, undo makernote specific settings
         virtual void visitIfdMakernoteEnd(TiffIfdMakernote* object);
-        //! Read an array entry component from the data buffer
-        virtual void visitArrayEntry(TiffArrayEntry* object);
-        //! Read an array element from the data buffer
-        virtual void visitArrayElement(TiffArrayElement* object);
+        //! Read a binary array from the data buffer
+        virtual void visitBinaryArray(TiffBinaryArray* object);
+        //! Read an element of a binary array from the data buffer
+        virtual void visitBinaryElement(TiffBinaryElement* object);
 
         //! Read a standard TIFF entry from the data buffer
         void readTiffEntry(TiffEntryBase* object);
@@ -602,9 +687,20 @@ namespace Exiv2 {
         //! Reset the state to the original state as set in the constructor.
         void resetState();
         //! Check IFD directory pointer \em start for circular reference
-        bool circularReference(const byte* start, uint16_t group);
+        bool circularReference(const byte* start, IfdId group);
         //! Return the next idx sequence number for \em group
-        int nextIdx(uint16_t group);
+        int nextIdx(IfdId group);
+
+        /*!
+          @brief Read deferred components.
+
+          This function is called after the TIFF composite is read by passing a
+          TiffReader to the accept() function of the root component. It reads
+          all components for which reading was deferred during that pass.  This
+          is usually done to make sure that all other components are accessible
+          at the time the deferred components are processed.
+         */
+        void postProcess();
         //@}
 
         //! @name Accessors
@@ -616,8 +712,9 @@ namespace Exiv2 {
         //@}
 
     private:
-        typedef std::map<const byte*, uint16_t> DirList;
+        typedef std::map<const byte*, IfdId> DirList;
         typedef std::map<uint16_t, int> IdxSeq;
+        typedef std::vector<TiffComponent*> PostList;
 
         // DATA
         const byte*          pData_;      //!< Pointer to the memory buffer
@@ -628,7 +725,8 @@ namespace Exiv2 {
         TiffRwState*         pOrigState_; //!< State class as set in the c'tor
         DirList              dirList_;    //!< List of IFD pointers and their groups
         IdxSeq               idxSeq_;     //!< Sequences for group, used for the entry's idx
-
+        PostList             postList_;   //!< List of components with deferred reading
+        bool                 postProc_;   //!< True in postProcessList()
     }; // class TiffReader
 
 }}                                      // namespace Internal, Exiv2
