@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      types.cpp
-  Version:   $Rev: 1750 $
+  Version:   $Rev: 2455 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: types.cpp 1750 2009-02-16 14:30:51Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: types.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -35,6 +35,9 @@ EXIV2_RCSID("@(#) $Id: types.cpp 1750 2009-02-16 14:30:51Z ahuggel $")
 #include "i18n.h"                               // for _exvGettext
 
 // + standard includes
+#ifdef EXV_UNICODE_PATH
+# include <windows.h> // for MultiByteToWideChar etc
+#endif
 #include <string>
 #include <iostream>
 #include <iomanip>
@@ -56,9 +59,15 @@ namespace {
         const char* name_;                      //!< Name of the type
         long size_;                             //!< Bytes per data entry
         //! Comparison operator for \em typeId
-        bool operator==(Exiv2::TypeId typeId) const;
+        bool operator==(Exiv2::TypeId typeId) const
+        {
+            return typeId_ == typeId;
+        }
         //! Comparison operator for \em name
-        bool operator==(const std::string& name) const;
+        bool operator==(const std::string& name) const
+        {
+            return 0 == strcmp(name_, name.c_str());
+        }
     }; // struct TypeInfoTable
 
     //! Lookup list with information of Exiv2 types
@@ -89,16 +98,6 @@ namespace {
         { Exiv2::langAlt,          "LangAlt",     1 }
     };
 
-    bool TypeInfoTable::operator==(Exiv2::TypeId typeId) const
-    {
-        return typeId_ == typeId;
-    }
-
-    bool TypeInfoTable::operator==(const std::string& name) const
-    {
-        return std::string(name_) == name;
-    }
-
 }
 
 // *****************************************************************************
@@ -153,8 +152,10 @@ namespace Exiv2 {
     {
         if (size > size_) {
             delete[] pData_;
-            size_ = size;
+            pData_ = 0;
+            size_ = 0;
             pData_ = new byte[size];
+            size_ = size;
         }
     }
 
@@ -178,6 +179,38 @@ namespace Exiv2 {
     // *************************************************************************
     // free functions
 
+    std::ostream& operator<<(std::ostream& os, const Rational& r)
+    {
+        return os << r.first << "/" << r.second;
+    }
+
+    std::istream& operator>>(std::istream& is, Rational& r)
+    {
+        int32_t nominator;
+        int32_t denominator;
+        char c('\0');
+        is >> nominator >> c >> denominator;
+        if (c != '/') is.setstate(std::ios::failbit);
+        if (is) r = std::make_pair(nominator, denominator);
+        return is;
+    }
+
+    std::ostream& operator<<(std::ostream& os, const URational& r)
+    {
+        return os << r.first << "/" << r.second;
+    }
+
+    std::istream& operator>>(std::istream& is, URational& r)
+    {
+        uint32_t nominator;
+        uint32_t denominator;
+        char c('\0');
+        is >> nominator >> c >> denominator;
+        if (c != '/') is.setstate(std::ios::failbit);
+        if (is) r = std::make_pair(nominator, denominator);
+        return is;
+    }
+
     uint16_t getUShort(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
@@ -236,6 +269,54 @@ namespace Exiv2 {
         return std::make_pair(nominator, denominator);
     }
 
+    float getFloat(const byte* buf, ByteOrder byteOrder)
+    {
+        // This algorithm assumes that the internal representation of the float
+        // type is the 4-byte IEEE 754 binary32 format, which is common but not
+        // required by the C++ standard.
+        assert(sizeof(float) == 4);
+        union {
+            uint32_t ul_;
+            float    f_;
+        } u;
+        u.ul_ = getULong(buf, byteOrder);
+        return u.f_;
+    }
+
+    double getDouble(const byte* buf, ByteOrder byteOrder)
+    {
+        // This algorithm assumes that the internal representation of the double
+        // type is the 8-byte IEEE 754 binary64 format, which is common but not
+        // required by the C++ standard.
+        assert(sizeof(double) == 8);
+        union {
+            uint64_t ull_;
+            double   d_;
+        } u;
+        u.ull_ = 0;
+        if (byteOrder == littleEndian) {
+            u.ull_ =   static_cast<uint64_t>(buf[7]) << 56
+                     | static_cast<uint64_t>(buf[6]) << 48
+                     | static_cast<uint64_t>(buf[5]) << 40
+                     | static_cast<uint64_t>(buf[4]) << 32
+                     | static_cast<uint64_t>(buf[3]) << 24
+                     | static_cast<uint64_t>(buf[2]) << 16
+                     | static_cast<uint64_t>(buf[1]) <<  8
+                     | static_cast<uint64_t>(buf[0]);
+        }
+        else {
+            u.ull_ =   static_cast<uint64_t>(buf[0]) << 56
+                     | static_cast<uint64_t>(buf[1]) << 48
+                     | static_cast<uint64_t>(buf[2]) << 40
+                     | static_cast<uint64_t>(buf[3]) << 32
+                     | static_cast<uint64_t>(buf[4]) << 24
+                     | static_cast<uint64_t>(buf[5]) << 16
+                     | static_cast<uint64_t>(buf[6]) <<  8
+                     | static_cast<uint64_t>(buf[7]);
+        }
+        return u.d_;
+    }
+
     long us2Data(byte* buf, uint16_t s, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
@@ -310,6 +391,55 @@ namespace Exiv2 {
         return o;
     }
 
+    long f2Data(byte* buf, float f, ByteOrder byteOrder)
+    {
+        // This algorithm assumes that the internal representation of the float
+        // type is the 4-byte IEEE 754 binary32 format, which is common but not
+        // required by the C++ standard.
+        assert(sizeof(float) == 4);
+        union {
+            uint32_t ul_;
+            float    f_;
+        } u;
+        u.f_ = f;
+        return ul2Data(buf, u.ul_, byteOrder);
+    }
+
+    long d2Data(byte* buf, double d, ByteOrder byteOrder)
+    {
+        // This algorithm assumes that the internal representation of the double
+        // type is the 8-byte IEEE 754 binary64 format, which is common but not
+        // required by the C++ standard.
+        assert(sizeof(double) == 8);
+        union {
+            uint64_t ull_;
+            double   d_;
+        } u;
+        u.d_ = d;
+        uint64_t m = 0xff;
+        if (byteOrder == littleEndian) {
+            buf[0] =  (byte)(u.ull_ & m);
+            buf[1] = (byte)((u.ull_ & (m <<  8)) >>  8);
+            buf[2] = (byte)((u.ull_ & (m << 16)) >> 16);
+            buf[3] = (byte)((u.ull_ & (m << 24)) >> 24);
+            buf[4] = (byte)((u.ull_ & (m << 32)) >> 32);
+            buf[5] = (byte)((u.ull_ & (m << 40)) >> 40);
+            buf[6] = (byte)((u.ull_ & (m << 48)) >> 48);
+            buf[7] = (byte)((u.ull_ & (m << 56)) >> 56);
+        }
+        else {
+            buf[0] = (byte)((u.ull_ & (m << 56)) >> 56);
+            buf[1] = (byte)((u.ull_ & (m << 48)) >> 48);
+            buf[2] = (byte)((u.ull_ & (m << 40)) >> 40);
+            buf[3] = (byte)((u.ull_ & (m << 32)) >> 32);
+            buf[4] = (byte)((u.ull_ & (m << 24)) >> 24);
+            buf[5] = (byte)((u.ull_ & (m << 16)) >> 16);
+            buf[6] = (byte)((u.ull_ & (m <<  8)) >>  8);
+            buf[7] =  (byte)(u.ull_ & m);
+        }
+        return 8;
+    }
+
     void hexdump(std::ostream& os, const byte* buf, long len, long offset)
     {
         const std::string::size_type pos = 8 + 16 * 3 + 2;
@@ -375,6 +505,32 @@ namespace Exiv2 {
 #endif
     }
 
+#ifdef EXV_UNICODE_PATH
+    std::string ws2s(const std::wstring& s)
+    {
+        int len;
+        int slength = (int)s.length() + 1;
+        len = WideCharToMultiByte(CP_ACP, 0, s.c_str(), slength, 0, 0, 0, 0);
+        char* buf = new char[len];
+        WideCharToMultiByte(CP_ACP, 0, s.c_str(), slength, buf, len, 0, 0);
+        std::string r(buf);
+        delete[] buf;
+        return r;
+    }
+
+    std::wstring s2ws(const std::string& s)
+    {
+        int len;
+        int slength = (int)s.length() + 1;
+        len = MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, 0, 0);
+        wchar_t* buf = new wchar_t[len];
+        MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, buf, len);
+        std::wstring r(buf);
+        delete[] buf;
+        return r;
+    }
+
+#endif // EXV_UNICODE_PATH
     template<>
     bool stringTo<bool>(const std::string& s, bool& ok)
     {
