@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,21 +20,22 @@
  */
 /*
   File:      value.cpp
-  Version:   $Rev: 1750 $
+  Version:   $Rev: 2455 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
              31-Jul-04, brad: added Time, Date and String values
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: value.cpp 1750 2009-02-16 14:30:51Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: value.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
 #include "value.hpp"
 #include "types.hpp"
 #include "error.hpp"
+#include "convert.hpp"
 
 // + standard includes
 #include <iostream>
@@ -75,8 +76,6 @@ namespace Exiv2 {
         case invalidTypeId:
         case signedByte:
         case unsignedByte:
-        case tiffFloat:
-        case tiffDouble:
             value = AutoPtr(new DataValue(typeId));
             break;
         case asciiString:
@@ -87,7 +86,7 @@ namespace Exiv2 {
             break;
         case unsignedLong:
         case tiffIfd:
-            value = AutoPtr(new ValueType<uint32_t>);
+            value = AutoPtr(new ValueType<uint32_t>(typeId));
             break;
         case unsignedRational:
             value = AutoPtr(new ValueType<URational>);
@@ -104,6 +103,12 @@ namespace Exiv2 {
         case signedRational:
             value = AutoPtr(new ValueType<Rational>);
             break;
+        case tiffFloat:
+            value = AutoPtr(new ValueType<float>);
+            break;
+        case tiffDouble:
+            value = AutoPtr(new ValueType<double>);
+            break;
         case string:
             value = AutoPtr(new StringValue);
             break;
@@ -152,6 +157,16 @@ namespace Exiv2 {
         return toString();
     }
 
+    long Value::sizeDataArea() const
+    {
+        return 0;
+    }
+
+    DataBuf Value::dataArea() const
+    {
+        return DataBuf(0, 0);
+    }
+
     DataValue::DataValue(TypeId typeId) : Value(typeId)
     {
     }
@@ -167,6 +182,11 @@ namespace Exiv2 {
     {
     }
 
+    long DataValue::count() const
+    {
+        return size();
+    }
+
     int DataValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // byteOrder not needed
@@ -178,12 +198,13 @@ namespace Exiv2 {
     {
         std::istringstream is(buf);
         int tmp;
-        value_.clear();
+        ValueType val;
         while (!(is.eof())) {
             is >> tmp;
             if (is.fail()) return 1;
-            value_.push_back(static_cast<byte>(tmp));
+            val.push_back(static_cast<byte>(tmp));
         }
+        value_.swap(val);
         return 0;
     }
 
@@ -209,7 +230,8 @@ namespace Exiv2 {
     {
         std::vector<byte>::size_type end = value_.size();
         for (std::vector<byte>::size_type i = 0; i != end; ++i) {
-            os << static_cast<int>(value_[i]) << " ";
+            os << static_cast<int>(value_[i]);
+            if (i < end - 1) os << " ";
         }
         return os;
     }
@@ -240,6 +262,26 @@ namespace Exiv2 {
         return Rational(value_[n], 1);
     }
 
+    StringValueBase::StringValueBase(TypeId typeId)
+        : Value(typeId)
+    {
+    }
+
+    StringValueBase::StringValueBase(TypeId typeId, const std::string& buf)
+        : Value(typeId)
+    {
+        read(buf);
+    }
+
+    StringValueBase::StringValueBase(const StringValueBase& rhs)
+        : Value(rhs), value_(rhs.value_)
+    {
+    }
+
+    StringValueBase::~StringValueBase()
+    {
+    }
+
     StringValueBase& StringValueBase::operator=(const StringValueBase& rhs)
     {
         if (this == &rhs) return *this;
@@ -271,6 +313,11 @@ namespace Exiv2 {
             );
     }
 
+    long StringValueBase::count() const
+    {
+        return size();
+    }
+
     long StringValueBase::size() const
     {
         return static_cast<long>(value_.size());
@@ -299,11 +346,39 @@ namespace Exiv2 {
         return Rational(value_[n], 1);
     }
 
+    StringValue::StringValue()
+        : StringValueBase(string)
+    {
+    }
+
+    StringValue::StringValue(const std::string& buf)
+        : StringValueBase(string, buf)
+    {
+    }
+
+    StringValue::~StringValue()
+    {
+    }
+
     StringValue* StringValue::clone_() const
     {
         return new StringValue(*this);
     }
 
+    AsciiValue::AsciiValue()
+        : StringValueBase(asciiString)
+    {
+    }
+
+    AsciiValue::AsciiValue(const std::string& buf)
+        : StringValueBase(asciiString, buf)
+    {
+    }
+
+    AsciiValue::~AsciiValue()
+    {
+    }
+
     int AsciiValue::read(const std::string& buf)
     {
         value_ = buf;
@@ -370,12 +445,21 @@ namespace Exiv2 {
                invalidCharsetId : charsetTable_[i].charsetId_;
     }
 
+    CommentValue::CommentValue()
+        : StringValueBase(Exiv2::undefined), byteOrder_(littleEndian)
+    {
+    }
+
     CommentValue::CommentValue(const std::string& comment)
-        : StringValueBase(Exiv2::undefined)
+        : StringValueBase(Exiv2::undefined), byteOrder_(littleEndian)
     {
         read(comment);
     }
 
+    CommentValue::~CommentValue()
+    {
+    }
+
     int CommentValue::read(const std::string& comment)
     {
         std::string c = comment;
@@ -389,30 +473,69 @@ namespace Exiv2 {
             charsetId = CharsetInfo::charsetIdByName(name);
             if (charsetId == invalidCharsetId) {
 #ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: " << Error(28, name) << "\n";
+                EXV_WARNING << Error(28, name) << "\n";
 #endif
                 return 1;
             }
             c.clear();
             if (pos != std::string::npos) c = comment.substr(pos+1);
         }
+        if (charsetId == unicode) {
+            const char* to = byteOrder_ == littleEndian ? "UCS-2LE" : "UCS-2BE";
+            convertStringCharset(c, "UTF-8", to);
+        }
         const std::string code(CharsetInfo::code(charsetId), 8);
         return StringValueBase::read(code + c);
     }
 
+    int CommentValue::read(const byte* buf, long len, ByteOrder byteOrder)
+    {
+        byteOrder_ = byteOrder;
+        return StringValueBase::read(buf, len, byteOrder);
+    }
+
+    long CommentValue::copy(byte* buf, ByteOrder byteOrder) const
+    {
+        std::string c = value_;
+        if (charsetId() == unicode) {
+            c = value_.substr(8);
+            std::string::size_type sz = c.size();
+            if (byteOrder_ == littleEndian && byteOrder == bigEndian) {
+                convertStringCharset(c, "UCS-2LE", "UCS-2BE");
+                assert(c.size() == sz);
+            }
+            else if (byteOrder_ == bigEndian && byteOrder == littleEndian) {
+                convertStringCharset(c, "UCS-2BE", "UCS-2LE");
+                assert(c.size() == sz);
+            }
+            c = value_.substr(0, 8) + c;
+        }
+        if (c.size() == 0) return 0;
+        assert(buf != 0);
+        return static_cast<long>(c.copy(reinterpret_cast<char*>(buf), c.size()));
+    }
+
     std::ostream& CommentValue::write(std::ostream& os) const
     {
-        CharsetId charsetId = this->charsetId();
-        if (charsetId != undefined) {
-            os << "charset=\"" << CharsetInfo::name(charsetId) << "\" ";
+        CharsetId csId = charsetId();
+        if (csId != undefined) {
+            os << "charset=\"" << CharsetInfo::name(csId) << "\" ";
         }
         return os << comment();
     }
 
-    std::string CommentValue::comment() const
+    std::string CommentValue::comment(const char* encoding) const
     {
-        if (value_.length() >= 8) return value_.substr(8);
-        return "";
+        std::string c;
+        if (value_.length() < 8) {
+            return c;
+        }
+        c = value_.substr(8);
+        if (charsetId() == unicode) {
+            const char* from = encoding == 0 || *encoding == '\0' ? detectCharset(c) : encoding;
+            convertStringCharset(c, from, "UTF-8");
+        }
+        return c;
     }
 
     CommentValue::CharsetId CommentValue::charsetId() const
@@ -425,6 +548,27 @@ namespace Exiv2 {
         return charsetId;
     }
 
+    const char* CommentValue::detectCharset(std::string& c) const
+    {
+        // Interpret a BOM if there is one
+        if (0 == strncmp(c.data(), "\xef\xbb\xbf", 3)) {
+            c = c.substr(3);
+            return "UTF-8";
+        }
+        if (0 == strncmp(c.data(), "\xff\xfe", 2)) {
+            c = c.substr(2);
+            return "UCS-2LE";
+        }
+        if (0 == strncmp(c.data(), "\xfe\xff", 2)) {
+            c = c.substr(2);
+            return "UCS-2BE";
+        }
+
+        // Todo: Add logic to guess if the comment is encoded in UTF-8
+
+        return byteOrder_ == littleEndian ? "UCS-2LE" : "UCS-2BE";
+    }
+
     CommentValue* CommentValue::clone_() const
     {
         return new CommentValue(*this);
@@ -483,7 +627,7 @@ namespace Exiv2 {
         std::ostringstream os;
         write(os);
         std::string s = os.str();
-        std::memcpy(buf, &s[0], s.size());
+        if (s.size() > 0) std::memcpy(buf, &s[0], s.size());
         return static_cast<long>(s.size());
     }
 
@@ -772,12 +916,16 @@ namespace Exiv2 {
         date_.day = day;
     }
 
+    DateValue::~DateValue()
+    {
+    }
+
     int DateValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // Hard coded to read Iptc style dates
         if (len != 8) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: " << Error(29) << "\n";
+            EXV_WARNING << Error(29) << "\n";
 #endif
             return 1;
         }
@@ -788,7 +936,7 @@ namespace Exiv2 {
                              &date_.year, &date_.month, &date_.day);
         if (scanned != 3) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: " << Error(29) << "\n";
+            EXV_WARNING << Error(29) << "\n";
 #endif
             return 1;
         }
@@ -800,7 +948,7 @@ namespace Exiv2 {
         // Hard coded to read Iptc style dates
         if (buf.length() < 8) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: " << Error(29) << "\n";
+            EXV_WARNING << Error(29) << "\n";
 #endif
             return 1;
         }
@@ -808,7 +956,7 @@ namespace Exiv2 {
                              &date_.year, &date_.month, &date_.day);
         if (scanned != 3) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: " << Error(29) << "\n";
+            EXV_WARNING << Error(29) << "\n";
 #endif
             return 1;
         }
@@ -834,6 +982,16 @@ namespace Exiv2 {
         return 8;
     }
 
+    const DateValue::Date& DateValue::getDate() const
+    {
+        return date_;
+    }
+
+    long DateValue::count() const
+    {
+        return size();
+    }
+
     long DateValue::size() const
     {
         return 8;
@@ -865,6 +1023,16 @@ namespace Exiv2 {
         return l;
     }
 
+    float DateValue::toFloat(long n) const
+    {
+        return static_cast<float>(toLong(n));
+    }
+
+    Rational DateValue::toRational(long n) const
+    {
+        return Rational(toLong(n), 1);
+    }
+
     TimeValue::TimeValue()
         : Value(time)
     {
@@ -882,6 +1050,10 @@ namespace Exiv2 {
         time_.tzMinute = tzMinute;
     }
 
+    TimeValue::~TimeValue()
+    {
+    }
+
     int TimeValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // Make the buffer a 0 terminated C-string for scanTime[36]
@@ -899,7 +1071,7 @@ namespace Exiv2 {
         if (rc) {
             rc = 1;
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: " << Error(30) << "\n";
+            EXV_WARNING << Error(30) << "\n";
 #endif
         }
         return rc;
@@ -919,7 +1091,7 @@ namespace Exiv2 {
         if (rc) {
             rc = 1;
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: " << Error(30) << "\n";
+            EXV_WARNING << Error(30) << "\n";
 #endif
         }
         return rc;
@@ -985,6 +1157,16 @@ namespace Exiv2 {
         return 11;
     }
 
+    const TimeValue::Time& TimeValue::getTime() const
+    {
+        return time_;
+    }
+
+    long TimeValue::count() const
+    {
+        return size();
+    }
+
     long TimeValue::size() const
     {
         return 11;
@@ -1021,4 +1203,14 @@ namespace Exiv2 {
         return result;
     }
 
+    float TimeValue::toFloat(long n) const
+    {
+        return static_cast<float>(toLong(n));
+    }
+
+    Rational TimeValue::toRational(long n) const
+    {
+        return Rational(toLong(n), 1);
+    }
+
 }                                       // namespace Exiv2
