@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      xmp.cpp
-  Version:   $Rev: 1769 $
+  Version:   $Rev: 2455 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   13-July-07, ahu: created
  */
 // *****************************************************************************
-#include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: xmp.cpp 1769 2009-03-20 15:30:01Z ahuggel $")
+#include "rcsid_int.hpp"
+EXIV2_RCSID("@(#) $Id: xmp.cpp 2455 2011-02-13 14:39:15Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -203,6 +203,11 @@ namespace Exiv2 {
         return TypeInfo::typeName(typeId());
     }
 
+    long Xmpdatum::typeSize() const
+    {
+        return 0;
+    }
+
     long Xmpdatum::count() const
     {
         return p_->value_.get() == 0 ? 0 : p_->value_->count();
@@ -376,8 +381,13 @@ namespace Exiv2 {
         if (!initialized_) {
 #ifdef EXV_HAVE_XMP_TOOLKIT
             initialized_ = SXMPMeta::Initialize();
-            SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/1.0/", "digiKam", 0);
-            SXMPMeta::RegisterNamespace("http://ns.microsoft.com/photo/1.0/", "MicrosoftPhoto", 0);
+            SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/1.0/", "digiKam");
+            SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/kipi/1.0/", "kipi");
+            SXMPMeta::RegisterNamespace("http://ns.microsoft.com/photo/1.0/", "MicrosoftPhoto");
+            SXMPMeta::RegisterNamespace("http://iptc.org/std/Iptc4xmpExt/2008-02-29/", "iptcExt");
+            SXMPMeta::RegisterNamespace("http://ns.useplus.org/ldf/xmp/1.0/", "plus");
+            SXMPMeta::RegisterNamespace("http://ns.iview-multimedia.com/mediapro/1.0/", "mediapro");
+            SXMPMeta::RegisterNamespace("http://ns.microsoft.com/expressionmedia/1.0/", "expressionmedia");
 #else
             initialized_ = true;
 #endif
@@ -397,23 +407,23 @@ namespace Exiv2 {
     }
 
 #ifdef EXV_HAVE_XMP_TOOLKIT
-    bool XmpParser::registerNs(const std::string& ns,
+    void XmpParser::registerNs(const std::string& ns,
                                const std::string& prefix)
     {
         try {
             initialize();
-            return SXMPMeta::RegisterNamespace(ns.c_str(), prefix.c_str(), 0);
+            SXMPMeta::DeleteNamespace(ns.c_str());
+            SXMPMeta::RegisterNamespace(ns.c_str(), prefix.c_str());
         }
         catch (const XMP_Error& e) {
             throw Error(40, e.GetID(), e.GetErrMsg());
         }
     } // XmpParser::registerNs
 #else
-    bool XmpParser::registerNs(const std::string& /*ns*/,
+    void XmpParser::registerNs(const std::string& /*ns*/,
                                const std::string& /*prefix*/)
     {
         initialize();
-        return true;
     } // XmpParser::registerNs
 #endif
 
@@ -439,7 +449,7 @@ namespace Exiv2 {
 
         if (!initialize()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "XMP Toolkit initialization failed.\n";
+            EXV_ERROR << "XMP toolkit initialization failed.\n";
 #endif
             return 2;
         }
@@ -556,13 +566,19 @@ namespace Exiv2 {
 
         return 0;
     }
-    catch (const XMP_Error& e) {
 #ifndef SUPPRESS_WARNINGS
-        std::cerr << Error(40, e.GetID(), e.GetErrMsg()) << "\n";
-#endif
+    catch (const XMP_Error& e) {
+        EXV_ERROR << Error(40, e.GetID(), e.GetErrMsg()) << "\n";
         xmpData.clear();
         return 3;
-    }} // XmpParser::decode
+    }
+#else
+    catch (const XMP_Error&) {
+        xmpData.clear();
+        return 3;
+    }
+#endif // SUPPRESS_WARNINGS
+    } // XmpParser::decode
 #else
     int XmpParser::decode(      XmpData&     xmpData,
                           const std::string& xmpPacket)
@@ -570,7 +586,7 @@ namespace Exiv2 {
         xmpData.clear();
         if (!xmpPacket.empty()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: XMP toolkit support not compiled in.\n";
+            EXV_WARNING << "XMP toolkit support not compiled in.\n";
 #endif
         }
         return 1;
@@ -590,11 +606,18 @@ namespace Exiv2 {
 
         if (!initialize()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "XMP Toolkit initialization failed.\n";
+            EXV_ERROR << "XMP toolkit initialization failed.\n";
 #endif
             return 2;
         }
-
+        // Register custom namespaces with XMP-SDK
+        for (XmpProperties::NsRegistry::iterator i = XmpProperties::nsRegistry_.begin();
+             i != XmpProperties::nsRegistry_.end(); ++i) {
+#ifdef DEBUG
+            std::cerr << "Registering " << i->second.prefix_ << " : " << i->first << "\n";
+#endif
+            registerNs(i->first, i->second.prefix_);
+        }
         SXMPMeta meta;
         for (XmpData::const_iterator i = xmpData.begin(); i != xmpData.end(); ++i) {
             const std::string ns = XmpProperties::ns(i->groupName());
@@ -628,7 +651,7 @@ namespace Exiv2 {
             }
             // Todo: Xmpdatum should have an XmpValue, not a Value
             const XmpValue* val = dynamic_cast<const XmpValue*>(&i->value());
-            assert(val);
+            if (val == 0) throw Error(52, i->key(), i->typeName());
             options =   xmpArrayOptionBits(val->xmpArrayType())
                       | xmpArrayOptionBits(val->xmpStruct());
             if (   i->typeId() == xmpBag
@@ -663,7 +686,7 @@ namespace Exiv2 {
                 continue;
             }
             // Don't let any Xmpdatum go by unnoticed
-            throw Error(38, i->tagName(), TypeInfo::typeName(i->typeId()));
+            throw Error(38, i->tagName(), i->typeName());
         }
         std::string tmpPacket;
         meta.SerializeToBuffer(&tmpPacket, xmpFormatOptionBits(static_cast<XmpFormatFlags>(formatFlags)), padding); // throws
@@ -671,12 +694,17 @@ namespace Exiv2 {
 
         return 0;
     }
-    catch (const XMP_Error& e) {
 #ifndef SUPPRESS_WARNINGS
-        std::cerr << Error(40, e.GetID(), e.GetErrMsg()) << "\n";
-#endif
+    catch (const XMP_Error& e) {
+        EXV_ERROR << Error(40, e.GetID(), e.GetErrMsg()) << "\n";
         return 3;
-    }} // XmpParser::decode
+    }
+#else
+    catch (const XMP_Error&) {
+        return 3;
+    }
+#endif // SUPPRESS_WARNINGS
+    } // XmpParser::decode
 #else
     int XmpParser::encode(      std::string& /*xmpPacket*/,
                           const XmpData&     xmpData,
@@ -685,7 +713,7 @@ namespace Exiv2 {
     {
         if (!xmpData.empty()) {
 #ifndef SUPPRESS_WARNINGS
-            std::cerr << "Warning: XMP toolkit support not compiled in.\n";
+            EXV_WARNING << "XMP toolkit support not compiled in.\n";
 #endif
         }
         return 1;
